Object subclass: #CqBoard
	instanceVariableNames: 'sectors stars '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqBoard commentStamp: '<historical>' prior: 0!
A simple data object that encapsulates the map.!


!CqBoard methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:04'!
at: aPoint
	^ sectors at: aPoint y - 1 * 15 + aPoint x! !

!CqBoard methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:52'!
starByName: aCharacter
	| index |
	index := aCharacter asUppercase asInteger - 64.
	^ (index between: 1 and: 20) ifTrue: [stars at: index]! !

!CqBoard methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:20'!
stars
	^ stars! !


!CqBoard methodsFor: 'initialization' stamp: 'sma 12/18/1999 20:23'!
initialize
	| position |
	sectors _ (1 to: 15 * 15) collect: [:i | CqSector new enemy: $ ; star: $.; tf: $ ].
	stars := (1 to: 20) collect: [:i | CqStar new name: (i + 64) asCharacter].
	1 to: stars size do: [:i | 
		[(self at: (position _ (CqGame rnd: 15) @ (CqGame rnd: 15))) star = $.] whileFalse.
		(stars at: i) position: position.
		(self at: position) enemy: $?; star: (stars at: i) name]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqBoard class
	instanceVariableNames: ''!

!CqBoard class methodsFor: 'instance creation' stamp: 'sma 12/11/1999 15:20'!
new
	^ super new initialize! !


Morph subclass: #CqConsole
	instanceVariableNames: 'textColor screenSize screenBuffer cursor showCursor blink inputQueue '
	classVariableNames: 'Font '
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqConsole commentStamp: '<historical>' prior: 0!
This morph implements a quick and dirty tty console.!


!CqConsole methodsFor: 'initialization' stamp: 'sma 4/28/2000 20:11'!
initialize
	super initialize.
	color := Color veryLightGray.
	textColor := Color black.
	cursor := 1 @ 1.
	showCursor := blink := true.
	self screenSize: 80 @ 25.
	inputQueue := SharedQueue new! !


!CqConsole methodsFor: 'accessing' stamp: 'sma 12/1/1999 15:34'!
cellSize
	^ 7 @ 15! !

!CqConsole methodsFor: 'accessing' stamp: 'sma 12/11/1999 14:32'!
cursor
	^ cursor! !

!CqConsole methodsFor: 'accessing' stamp: 'sma 12/11/1999 13:59'!
screenSize
	^ screenSize! !

!CqConsole methodsFor: 'accessing' stamp: 'sma 4/28/2000 20:07'!
screenSize: aPoint
	screenSize := aPoint.
	screenBuffer := String new: screenSize x * screenSize y withAll: $ .
	self extent: screenSize * self cellSize.
	self goto: 1 @ 1! !

!CqConsole methodsFor: 'accessing' stamp: 'sma 4/28/2000 20:06'!
showCursor
	^ showCursor! !

!CqConsole methodsFor: 'accessing' stamp: 'sma 4/28/2000 20:06'!
showCursor: aBoolean
	showCursor = aBoolean
		ifFalse:
			[showCursor := aBoolean.
			blink := true.
			self invalidCursor]! !


!CqConsole methodsFor: 'drawing' stamp: 'sma 4/28/2000 20:07'!
drawOn: aCanvas
	| box clipRect cellSize cellOrigin cellCorner p |
	super drawOn: aCanvas.
	box := self bounds.
	clipRect := aCanvas clipRect.
	cellSize := self cellSize.
	cellOrigin := clipRect origin - box origin // cellSize max: 0 @ 0.
	cellCorner := (clipRect corner - box origin + cellSize - 1 // cellSize min: screenSize) - 1.
	cellOrigin y to: cellCorner y do: [:y |
		cellOrigin x to: cellCorner x do: [:x |
			p := x @ y * cellSize + box origin.
			(showCursor and: [blink and: [cursor x - 1 = x and: [cursor y - 1 = y]]])
				ifTrue:
					[aCanvas
						fillRectangle: (p extent: cellSize)
						color: Color gray].
			aCanvas
				stencil: Font
				at: p
				sourceRect: ((self screenAt: x @ y + 1) asInteger - 32 @ 0 * cellSize extent: cellSize)
				color: textColor]]
! !

!CqConsole methodsFor: 'drawing' stamp: 'sma 4/28/2000 20:07'!
step
	blink := blink not.
	showCursor ifTrue: [self invalidCursor]! !

!CqConsole methodsFor: 'drawing' stamp: 'sma 4/28/2000 20:07'!
stepTime
	^ 400! !


!CqConsole methodsFor: 'event handling' stamp: 'sma 12/11/1999 14:27'!
handlesMouseOver: event
	"Enable #mouseEnter: and #mouseLeave: events."

	^ true
! !

!CqConsole methodsFor: 'event handling' stamp: 'sma 12/11/1999 14:01'!
keyStroke: evt
	inputQueue nextPut: evt keyCharacter! !

!CqConsole methodsFor: 'event handling' stamp: 'sma 4/28/2000 20:08'!
mouseEnter: event
	"Grab keyboard focus."

	event hand newKeyboardFocus: self.
	self showCursor: true.
	super mouseEnter: event
! !

!CqConsole methodsFor: 'event handling' stamp: 'sma 4/28/2000 20:08'!
mouseLeave: event
	"Ungrab keyboard focus."

	super mouseLeave: event.
	self showCursor: false.
	event hand newKeyboardFocus: nil! !


!CqConsole methodsFor: 'reading' stamp: 'sma 12/11/1999 14:01'!
nextChar
	^ inputQueue next! !


!CqConsole methodsFor: 'writing' stamp: 'sma 12/11/1999 14:30'!
clear
	"Clear screen."

	self screenSize: self screenSize.
	self goto: 1@1.
	self changed! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/11/1999 14:57'!
cr
	self putChar: Character cr! !

!CqConsole methodsFor: 'writing' stamp: 'sma 4/28/2000 20:03'!
goto: aPoint
	"Set cursor position."

	showCursor
		ifTrue:
			[blink ifTrue: [self invalidCursor].
			cursor := aPoint.
			blink := true.
			self invalidCursor]
		ifFalse: [cursor := aPoint]! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/11/1999 14:57'!
move: aPoint
	"Adjust cursor position."

	self goto: cursor + aPoint! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/18/1999 14:49'!
print: aStringOrObject
	"Deprecated."

	(aStringOrObject isKindOf: String)
		ifTrue: [aStringOrObject do: [:each | self putChar: each]]
		ifFalse: [self print: aStringOrObject asString]! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/1/1999 19:48'!
printf: fmtString
	self print: fmtString! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/1/1999 19:47'!
printf: fmtString with: arg
	self printf: fmtString withArguments: (Array with: arg)! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/1/1999 19:47'!
printf: fmtString with: arg1 with: arg2
	self printf: fmtString withArguments: (Array with: arg1 with: arg2)! !

!CqConsole methodsFor: 'writing' stamp: 'sma 12/11/1999 23:20'!
printf: fmtString withArguments: arguments
	"An ad hoc printf() implementation which supports formatted
	strings (%s) and integers (%d), optionally with leading zeros."

	| s a ch pad length value neg |
	s := ReadStream on: fmtString.
	a := 0.
	[s atEnd] whileFalse: [
		ch := s next.
		ch = $%
			ifTrue:
				[ch := s next.
				pad := $ .
				length := 0.
				(neg := ch = $-) ifTrue: [ch := s next].
				ch isDigit 
 					ifTrue:
						[ch = $0 ifTrue: [pad := $0].
						[ch isDigit]
							whileTrue: [length := length * 10 + ch digitValue. ch := s next]].
				ch = $s ifTrue: [
					value := arguments at: (a := a + 1).
					(value isKindOf: String) ifFalse: [value := value asString].
					pad := $ .
					length := length - value size.
					neg ifTrue: [self printf: value].
					[length > 0] whileTrue: [
						self putChar: pad.
						length := length - 1].
					neg ifFalse: [self printf: value]]
				ifFalse: [ch = $d ifTrue: [
					value := arguments at: (a := a + 1).
						(value < 0 and: [pad = $0]) ifTrue: [
							self putChar: $-.
							length := length - 1.
							value := value abs].
						value := value printString.
						length := length - value size.
						[length > 0] whileTrue: [
							self putChar: pad.
							length := length - 1].
						self print: value]
				ifFalse: [ch = $c ifTrue: [
					self putChar: (arguments at: (a := a + 1))]
				ifFalse: [self putChar: ch]]]]
			ifFalse: [self putChar: ch]]! !

!CqConsole methodsFor: 'writing' stamp: 'sma 4/28/2000 20:12'!
putChar: aCharacter
	aCharacter = Character backspace ifTrue: [^ self move: -1 @ 0].
	aCharacter = Character cr ifTrue: [^ self goto: 1 @ (cursor y + 1)].
	self screenAt: cursor put: aCharacter.
	self invalidCursor.
	self move: 1 @ 0! !


!CqConsole methodsFor: 'private' stamp: 'sma 12/11/1999 14:30'!
invalidCell: aPoint
	self invalidRect: (self bounds origin + (aPoint - 1 * self cellSize) extent: self cellSize)! !

!CqConsole methodsFor: 'private' stamp: 'sma 4/28/2000 20:04'!
invalidCursor
	self invalidCell: self cursor! !

!CqConsole methodsFor: 'private' stamp: 'sma 12/11/1999 14:06'!
screenAt: aPoint
	^ screenBuffer at: aPoint y - 1 * screenSize x + aPoint x! !

!CqConsole methodsFor: 'private' stamp: 'sma 12/11/1999 14:06'!
screenAt: aPoint put: aCharacter
	^ screenBuffer at: aPoint y - 1 * screenSize x + aPoint x put: aCharacter! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqConsole class
	instanceVariableNames: ''!

!CqConsole class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 10:57'!
run: aGame
	| con |
	(con := self new) openInWorld.
	[[aGame runOnConsole: con] ensure: [con delete]] fork.
	^ aGame! !


!CqConsole class methodsFor: 'class initialization' stamp: 'sma 12/1/1999 15:29'!
initialize
	Font := (Form
	extent: 672@15
	depth: 1
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 939524096 0 0 0 0 0 0 0 0 0 0 0 0 0 2207905 67110920 1077936128 145504 3787507591 1043886080 0 1147396591 530554606 4176967453 4221335667 3786407419 4157070833 3288794112 268632064 3221684416 541459200 0 0 0 1090781310 2171043 2315257864 1106280448 148513 304169992 574916608 12595212 1143018020 2290655812 539107853 2569151625 308881553 574916881 33620992 134283264 1074266176 131328 0 131072 0 2164392002 2171204 67504144 541097984 279584 270615568 38045888 3238004754 1280381444 1242177604 539116045 2569151625 302057617 573067297 33622528 7430627 1193220952 3789742365 907860077 2984761787 4157464048 2164392002 2098148 25692176 547389440 279584 549603102 70813888 3322906114 1414652420 1309548668 539132426 2502042761 297863306 705712193 33624320 9015844 3364365924 539115786 2569168024 3257010321 575179040 2164392002 2097475 2382888976 547876879 2148025376 1076117649 71596032 134217988 1414599172 1242179396 555975178 2502047881 3760261258 705699905 16842752 7934468 1334321732 539197706 2434942088 2176977041 707807296 2164394306 996 2165637136 536903680 541728 2150097041 71567360 101680648 1291391492 1208492612 555893320 2502041737 539035786 707272833 16842752 8983044 1208492612 539132170 2434942088 2149714058 707809409 16848450 327 43122704 536904064 202392609 304105617 138678464 3238004736 1082724900 2290623044 555885128 2468487305 320931972 709107985 8454144 8983076 1208492612 539115786 2434942088 2183276938 709374224 2164392002 2097793 17629200 536903936 202389753 4055951118 137945281 2160078872 1172300239 530324718 4173805533 4212945011 2328069892 348549617 8454144 8380867 3885965038 4163299295 4221335673 4089562820 348922352 2164392062 641 8 1073742592 2097152 0 1 0 939524096 0 0 56 0 1 8454144 0 512 2097152 16392 0 8192 2164391936 0 8 1073742336 0 0 0 0 0 0 0 0 0 1 3221684224 0 7168 29360128 57372 0 61440 1074003968 0 0 0 0 0 0 0 0 0 0 0 0 0 127 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !


Object subclass: #CqGame
	instanceVariableNames: 'console gameOver turn productionYear stars board player enemy enResearch '
	classVariableNames: 'Dice '
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqGame commentStamp: 'sma 4/29/2000 10:40' prior: 0!
This is the CONQUEST game implemented as a big blob of code which was directly translated from a kind-of ancient C version I found on an old Amiga PD disk.  I believe, the C code was cross compiled from Pascal.  Use

CqGame run

to start the game.  Goal of the game is to conquer the galaxy before the ENEMY does.  Destory the ENEMY or accumulate eight times more population.!


!CqGame methodsFor: 'combat' stamp: 'sma 12/18/1999 14:02'!
battle
	| first |
	first := true.
	stars do: [:star |
		((enemy tfsAtStar: star) size > 0
		and: [(player tfsAtStar: star) size > 0
		and: [(enemy anyBc: star) or: [player anyBc: star]]]) ifTrue: [
			first ifTrue: [
				self point: 33@20.
				self printf: '* Tf battle *   '.
				first := false].
			self tfBattle: star].
		"By now we know there is only one side left with warships here"
		((enemy anyBc: star) and: [(player coloniesAtStar: star) > 0])
			ifTrue: [self enemyAttack: star].
		((player anyBc: star) and: [(enemy coloniesAtStar: star) > 0])
			ifTrue: [self playerAttack: star]]! !

!CqGame methodsFor: 'combat' stamp: 'sma 12/18/1999 14:31'!
blast: planet with: factors
	| killed |
	killed := planet capacity min: factors // 4.
	planet inhabitants: (planet inhabitants min: planet capacity) - killed.
	planet iu: (planet iu - killed min: planet inhabitants * 2).
	planet capacity: planet capacity - killed.
	
	"Totally destroyed..."
	planet inhabitants <= 0 ifTrue: [
		planet inhabitants: 0.
		planet iu: 0.
		planet mb: 0.
		planet amb: 0.
		planet team notNil ifTrue: [
			planet team: nil.
			planet eSeeTeam: nil.
			planet conquered: false.
			self onBoard: planet star position]]! !

!CqGame methodsFor: 'combat' stamp: 'sma 12/19/1999 15:03'!
checkGameOver
	| quit transports inhabs enTotal plTotal enDead plDead f |
	quit := gameOver.
	transports := Dictionary new at: enemy put: 0; at: player put: 0; yourself.
	inhabs := Dictionary new at: enemy put: 0; at: player put: 0; yourself.
	
	(Array with: enemy with: player) do: [:each |
		each tfs do: [:tf |
			tf exists ifTrue: [transports at: each put: (transports at: each) + tf t]]].
	stars do: [:star |
		star planets do: [:planet |
			planet team notNil
				ifTrue:
					[inhabs
						at: planet team 
						put: (inhabs at: planet team) + planet iu]]].

	enTotal := (inhabs at: enemy) + (transports at: enemy).
	plTotal := (inhabs at: player) + (transports at: player).
	enDead := enTotal = 0.
	plDead := plTotal = 0.

	(plDead not and: [enDead not and: [turn >= 40]]) ifTrue: [
		f := enTotal / plTotal.
		f >= 8 ifTrue: [plDead := true].
		1/f >= 8 ifTrue: [enDead := true]].
	
	gameOver := gameOver | enDead | plDead.
	gameOver ifTrue: [
		console clear;
		 printf: '*** Game over ***';
		 cr;
		 cr;
		 printf: 'Player: Population in transports:%4d' with: (transports at: player);
		 printf: '  IU''s on colonies:%4d  TOTAL:%4d' with: (inhabs at: player) with: plTotal;
		 cr;
		 printf: 'Enemy:  Population in transports:%4d' with: (transports at: enemy);
		 printf: '  IU''s on colonies:%4d  TOTAL:%4d' with: (inhabs at: enemy) with: enTotal;
		 cr;
		 cr.
	enTotal > plTotal | quit ifTrue: [
		console printf: '**** THE ENEMY HAS CONQUERED THE GALAXY ***']
	ifFalse: [plTotal > enTotal ifTrue: [
		console printf: '*** PLAYER WINS - YOU HAVE SAVED THE GALAXY!! ***']
	ifFalse: [
		console printf: '*** DRAWN ***']].
	self getChar]! !

!CqGame methodsFor: 'combat' stamp: 'sma 12/19/1999 15:00'!
fireSalvo: attTeam tf: tf planet: planet firstTime: firstTime
	| defTeam attForces defForces attOdds attSave defOdds defSave aLosses pLosses bases |
	self point: 1@24.
	self printf: self blankLine.
	defTeam := attTeam = enemy ifTrue: [player] ifFalse: [enemy].

	attForces := attTeam weaponsResearch * (tf c * 6 + (tf b * 40)).
	defForces := defTeam weaponsResearch * (planet mb * 6 + (planet amb * 40)).

	defForces > 0 ifTrue: [
		attOdds := (defForces / attForces) asFloat min: 14.0.
		attSave := ((0.8 log * attOdds) exp * 100) asInteger.
		defOdds := (attForces / defForces) asFloat min: 14.0.
		defSave := ((0.8 log * defOdds) exp * 100) asInteger.

		self point: 1@20.
		attTeam = player
			ifTrue: [self printf: 'TF%c' with: tf name]
			ifFalse: [self printf: ' EN'].
		self printf: ': %4d(weap %2d)' with: attForces with: attTeam weaponsResearch.
		self printf: 'sur: %3d.0' with: attSave.
		
		self point: 1@21.
		self printf: ' %c%d:' with: planet star name with: planet number.
		self printf: ' %4d(weap %2d)' with: defForces with: defTeam weaponsResearch.
		self printf: 'sur: %3d.0' with: defSave.
		
		self point: 1@22.
		self printf: 'Attacker losses:              '.
		self point: 1@23.
		self printf: '  Planet losses:              '.
		
		aLosses := pLosses := false.
		[self point: 17@22.
		aLosses := aLosses | (self lose: (CqRef object: tf aspect: #c) ofType: $c chance: attSave).
		aLosses := aLosses | (self lose: (CqRef object: tf aspect: #b) ofType: $b chance: attSave).

		self point: 17@23.
		bases := planet mb.
		pLosses := pLosses | (self lose: (CqRef object: planet aspect: #mb) ofType: $m chance: attSave).
		bases = planet mb ifFalse: [self printf: 'b'].
		bases := planet amb.
		pLosses := pLosses | (self lose: (CqRef object: planet aspect: #amb) ofType: $a chance: attSave).
		bases = planet amb ifFalse: [self printf: 'mb'].
		firstTime not and: [pLosses not and: [aLosses]]] whileTrue.
	
		aLosses ifFalse: [self point: 17@22; printf: '(none)'].
		pLosses ifFalse: [self point: 17@23; printf: '(none)']].

	(planet mb + planet amb = 0 and: [attTeam anyBc: planet star]) ifTrue: [
		self point: 1@24.
		self printf: 'Planet %d falls!!               ' with: planet number.
		planet team: attTeam.
		planet eSeeTeam: attTeam.
		planet conquered: true.
		self point: 50@1.
		self printStar: planet star.
		self clearField.
		self onBoard: tf dest position]! !

!CqGame methodsFor: 'combat' stamp: 'sma 12/19/1999 10:55'!
playSalvo: star
	| num planet tf firstTime |
	self printf: 'Attack planet '.
	num := enemy coloniesAtStar: star.
	num = 0 ifTrue: [self errorMsg: '!! no enemy colony present'. ^ true].
	num > 1 ifTrue: [
		self printf: ':'.
		num := self getChar asciiValue - 48.
		planet := star planets detect: [:p | p number = num]
			ifNone: [self errorMsg: '!! That is not a useable planet'. ^ true].
		planet team = enemy ifFalse: [self errorMsg: ' !!Not an enemy colony'. ^ true]]
	ifFalse: [
		planet := star planets detect: [:p | p team = enemy].
		self printf: '%d' with: planet number.
		self clearLeft].

	"Find out which tf is attacking"
	self point: 1@19.
	self printf: ' attacking tf '.
	(player tfsAtStar: star) size > 1 ifTrue: [
		self printf: ':'.
		tf := player tfByName: self getChar.
		tf isNil ifTrue: [self errorMsg: ' !!Illegal tf'. ^ true].
		tf exists ifFalse: [self errorMsg: ' !!Nonexistent tf'. ^ true].
		(tf dest ~= star or: [tf eta ~= 0]) ifTrue: [self errorMsg: ' !!Tf is not at this star      '. ^ true].
		tf b + tf c = 0 ifTrue: [self errorMsg: ' !!Tf has no warships'. ^ true]]
	ifFalse: [
		tf := (player tfsAtStar: star) first.
		self printf: '%c' with: tf name].

	firstTime := planet underAttack not.
	firstTime ifTrue: [
		planet underAttack: true.
		self point: 1@50.
		self printStar: star.
		self clearField].
	self fireSalvo: player tf: tf planet: planet firstTime: firstTime.
	self zeroTf: tf team: player.
	
	^ (enemy coloniesAtStar: star) > 0 and: [player anyBc: star]! !

!CqGame methodsFor: 'combat' stamp: 'sma 12/18/1999 14:33'!
playerAttack: star
	"Player attacks specified star. Do combat."

	| battle |
	self point: 33@20.
	self printf: 'Attack at star %c' with: star name.

	battle := true.
	[battle] whileTrue: [
		self point: 50@1.
		self printStar: star.
		self clearField.
		self point: 1@18.
		self printf: 'P?                            '.
		self point: 3@18.
		
		self getChar caseOf: {
			[$S] ->	[self starSum].
			[$M] ->	[self printMap].
			[$H] ->	[self help: 3; pause].
			[$N] ->	[self newTf].
			[$J] ->	[self joinTf].
			[$C] ->	[self printCol].
			[$R] ->	[self resSum].
			[$T] ->	[self tfSum].
			[$G] ->	[self playSalvo: star].
			[$ ] ->	[self playSalvo: star].
			[$B] ->	[console printf: 'break off attack'. battle := false]
		} otherwise: [
			self clearLeft.
			self errorMsg: ' !!Illegal command']].

	star planets do: [:each | each underAttack: false].
	
	self point: 1@24.
	self printf: 'Planet attack concluded       '.
	
	self revolt: star! !

!CqGame methodsFor: 'combat' stamp: 'sma 4/29/2000 11:12'!
tfBattle: star
	| enTf tfs plTf args battle enOdds plOdds first plLoss enLoss newTf slist dstar fin |
	(board at: star position) enemy: $!!.
	self updateBoard: star position on: #left.

	enTf := (enemy tfsAtStar: star) first.
	tfs := player tfsAtStar: star.
	tfs size > 1 ifTrue: [
		2 to: tfs size do: [:i | self joinsilent: tfs first with: (tfs at: i)]].
	plTf := tfs first.
	
	args := Array new: 2.
 	battle := self displayForces: enTf and: plTf odds: args.
	enOdds := args first.
	plOdds := args last.
	
	self pause.
	
	first := true.
	[battle] whileTrue: [
		plLoss := false.
		enLoss := false.
		self point: 1@21.
		self printf: ' Enemy losses:                '.
		self point: 1@22.
		self printf: 'Player losses:                '.
		
		"Fight until there are losses, but not first time"
		[self point: 15@21.
		enLoss := enLoss | (self lose: (CqRef object: enTf aspect: #t) ofType: $t chance: enOdds).
		enLoss := enLoss | (self lose: (CqRef object: enTf aspect: #s) ofType: $s chance: enOdds).
 		enLoss := enLoss | (self lose: (CqRef object: enTf aspect: #c) ofType: $c chance: enOdds).
		enLoss := enLoss | (self lose: (CqRef object: enTf aspect: #b) ofType: $b chance: enOdds).
 
		self point: 15@22.
		plLoss := plLoss | (self lose: (CqRef object: plTf aspect: #t) ofType: $t chance: plOdds).
		plLoss := plLoss | (self lose: (CqRef object: plTf aspect: #s) ofType: $s chance: plOdds).
 		plLoss := plLoss | (self lose: (CqRef object: plTf aspect: #c) ofType: $c chance: plOdds).
		plLoss := plLoss | (self lose: (CqRef object: plTf aspect: #b) ofType: $b chance: plOdds).

		first not and: [enLoss not and: [plLoss not]]] whileTrue.
		first := false.
 
		enLoss ifFalse: [self point: 15@21; printf: '(none)'].
		plLoss ifFalse: [self point: 15@22; printf: '(none)'].

		battle := self displayForces: enTf and: plTf odds: args.
		enOdds := args first.
		plOdds := args last.
		
		battle ifTrue: [
			"Withdraw the bad guys"
			newTf := enemy getTf: star.
			plTf c + plTf b > 0 ifTrue: [
				newTf t: enTf t.
				newTf s: enTf s.
				(self bestWithdrawPlan: star odds: enOdds) ifTrue: [
					newTf c: enTf c.
					newTf b: enTf b]].
			"If any need to be withdrawn"
			newTf t + newTf s + newTf c + newTf b > 0 ifTrue: [
				"Find a place to withdraw to"
				slist := Array new: stars size.
				self getStars: star into: slist.
				[i := CqGame rnd: stars size.
				(slist at: i) > 0] whileFalse.
				dstar := stars at: i.
				newTf dest: dstar.
				newTf eta: ((slist at: i) - 0.01 / enemy velocityResearch) asInteger + 1.
				newTf positionf: dstar position]
			ifFalse: [newTf dest: nil].
			
			fin := false.
			[self point: 1@18.
			self printf: 'B?                            '.
			self point: 3@18.
			self getChar caseOf: {
				[$M] -> [self printMap].
				[$H] -> [self help: 2].
				[$S] -> [self starSum].
				[$T] -> [self tfSum].
				[$C] -> [self printCol].
				[$?] -> [self help: 2].
				[$R] -> [self resSum].
				[$O] -> [
					battle := self displayForces: enTf and: plTf odds: args.
					enOdds := args first.
					plOdds := args last].
				[$W] -> [
					self withdraw: star tf: plTf. 
					battle := self displayForces: enTf and: plTf odds: args.
					enOdds := args first.
					plOdds := args last].
				[$ ] -> [fin := true].
				[$G] -> [fin := true]
			} otherwise: [self printf: '!!illegal command'].
			fin not and: [battle]] whileTrue.

			self zeroTf: newTf team: enemy.
			self zeroTf: plTf team: player.

			newTf dest notNil ifTrue: [
				self point: 1@23.
				self printf: 'en withdraws'.
				self point: 14@23.
				self dispTf: newTf.

				enTf t: enTf t - newTf t.
				enTf s: enTf s - newTf s.
				enTf c: enTf c - newTf c.
				enTf b: enTf b - newTf b.
				self zeroTf: enTf team: enemy.

				battle := self displayForces: enTf and: plTf odds: args.
				enOdds := args first.
				plOdds := args last]]].
	self zeroTf: plTf team: player.
	self zeroTf: enTf team: enemy.

	self revolt: star.
	self onBoard: star position! !

!CqGame methodsFor: 'combat' stamp: 'sma 12/18/1999 22:39'!
withdraw: star tf: tf
	"Private. Withdraw from battle."

	| withTf |
	self printf: 'ithdraw '.
	self clearLeft.
	self point: 1@19.
	withTf := self splitTf: tf.
	withTf exists ifTrue: [
		self point: 1@20.
		(self setDes: withTf)
			ifTrue: [tf dest: star. self joinsilent: tf with: withTf]
			ifFalse: [tf withdrew: true]]! !


!CqGame methodsFor: 'commands' stamp: 'sma 12/18/1999 22:24'!
blastPlanet
	| tf star planet num factors args amt |
	self printf: 'last'.
	self clearLeft.
	self point: 1@19.
	self printf: 'Firing TF:'.
	tf := player tfByName: self getChar.
	tf isNil ifTrue: [^ self errorMsg: ' !!Illegal tf'].
	tf exists ifFalse: [^ self errorMsg: ' !!Nonexistent tf'].
	tf eta ~= 0 ifTrue: [^ self errorMsg: ' !!Tf is not in normal space'].
	tf isBlasting ifTrue: [^ self errorMsg: ' !!Tf is already blasting'].
	tf c + tf b = 0 ifTrue: [^ self errorMsg: ' !!Tf has no warships'].

	star := tf dest.
	star planets isEmpty ifTrue: [^ self errorMsg: ' !!No planets at star %c' with: star name].
	self point: 1@20.
	self printf: 'Target colony '.
	star planets size = 1 ifTrue: [
		planet := star planets first.
		self printf: '%d' with: planet number]
	ifFalse: [
		self printf: ':'.
		num := self getChar asInteger - 48.
		planet := star planets detect: [:p | p number = num]
			ifNone: [^ self errorMsg: ' !!No such planet at this star']].

	planet team = enemy ifTrue: [^ self errorMsg: ' !!Conquer it first!!'].
	(planet team = player and: [planet conquered not])
		ifTrue: [^ self errorMsg: ' !!That is a human colony!!!!'].

	factors := tf c * 7 + (tf b * 40) * player weaponsResearch.

	self point: 1@21.
	self printf: 'Units (max %3d) :' with: factors // 4.
	self point: 18@21.
	args := Array with: self getLine with: 0.
	self getToken: args.
	amt := args last.
	amt < 0 ifTrue: [factors := 0].
	amt > 0 ifTrue: [factors := factors min: amt * 4].

	tf isBlasting: true.
	self point: 1@22.
	self printf: 'Blasting %3d units' with: factors // 4.

	self blast: planet with: factors.

	self point: 1@23.
	self printf: '%c' with: star name.
	planet pSeeCapacity: planet capacity.
	self printPlanet: planet see: true
! !

!CqGame methodsFor: 'commands' stamp: 'sma 12/12/1999 00:18'!
inputPlayer
	| fin key |

	self point: 33@20.
	self printf: '* Movement *    '.

	fin := false.
	[self point: 1@18.
	self printf: '?                             '.
	self point: 2@18.
	key := self getChar.
	key caseOf: {
		[$M] ->	[self printMap].
		[$B] ->	[self blastPlanet].
		[$G] ->	[fin := true].
		[$ ] ->	[fin := true].
		[$H] ->	[self help: 1].
		[$L] ->	[self land].
		[$D] ->	[self sendTf].
		[$S] ->	[self starSum].
		[$N] ->	[self newTf].
		[$J] ->	[self joinTf].
		[$C] ->	[self printCol].
		[$R] ->	[self resSum].
		[$T] ->	[self tfSum].
		[$Q] ->	[self quit. fin := gameOver]
	} otherwise: [self errorMsg: '  !! illegal command'].
	fin] whileFalse
! !

!CqGame methodsFor: 'commands' stamp: 'sma 12/19/1999 10:36'!
investPlayer: star planet: planet
	| tf printtf balance args cost key amt |
	tf := player getTf: star.
	printtf := false.
	balance := planet iu.

	self clearLeft.
	self point: 1@19.
	self printf: '%c' with: star name.
	self printf: '%d:%2d                         '
		with: planet number
		with: planet pSeeCapacity.
	self point: 6@19.
	self printf: '(%2d,/%3d)'
		with: planet inhabitants
		with: planet iu.
	self printf: (planet conquered ifTrue: ['Con'] ifFalse: ['   ']).
	planet mb > 0
		ifTrue: [self printf: '%2dmb' with: planet mb]
		ifFalse: [self printf: '    '].
	planet amb > 0
		ifTrue: [self printf: '%2damb' with: planet amb].
	self point: 1@20.

  	[self point: 1@18.
	self printf: '%3d?                          ' with: balance.
	self point: 5@18.
	args := Array with: self getLine with: 0.

 	[cost := 0.
	key := self getToken: args.
	amt := args last.
	key caseOf: {
		[$A] -> [
			planet inhabitants = 0
				ifTrue: [self errorMsg: '  !!abandoned planet']
			ifFalse: [
				planet conquered
					ifTrue: [self errorMsg: ' !!No amb  on conquered colony']
				ifFalse: [
					cost := amt * 35.
					cost <= balance ifTrue: [planet amb: planet amb + amt]]]].
		[$B] -> [cost := amt * 70. cost <= balance ifTrue: [tf b: tf b + amt. printtf := true]].
		[$C] -> [cost := amt * 16. cost <= balance ifTrue: [tf c: tf c + amt. printtf := true]].
		[$H] -> [self help: 4].
		[$M] -> [
			planet inhabitants = 0
				ifTrue: [self errorMsg: '  !!abandoned planet']
				ifFalse: [
					planet conquered
						ifTrue: [self errorMsg: ' !!No mb  on conquered colony']
						ifFalse: [
							cost := amt * 8.
							cost <= balance ifTrue: [planet mb: planet mb + amt]]]].
		[$S] -> [cost := amt * 6. cost <= balance ifTrue: [tf s: tf s + amt. printtf := true]].
		[$T] -> [
			planet conquered
				ifTrue: [self errorMsg: '!!No transports on conqered col']
			ifFalse: [
				amt > planet inhabitants
					ifTrue: [self errorMsg: ' !! Not enough people for trans']
				ifFalse: [
					cost := amt. 
					cost <= balance ifTrue: [
						planet inhabitants: planet inhabitants - amt.
						planet iu: (planet iu - amt min: planet inhabitants * 2).
						planet inhabitants = 0 ifTrue: [
							planet team: nil; amb: 0; mb: 0; iu: 0.
							(player coloniesAtStar: star) = 0 ifTrue: [
								(board at: star position) enemy: $ .
								self updateBoard: star position on: #left]].
						tf t: tf t + amt.
						printtf := true]]]].
		[$I] -> [
			planet iu + amt > (planet inhabitants * 2)
				ifTrue: [self errorMsg: ' !!Can''t support that many iu''s']
				ifFalse: [
					cost := amt * 3.
					cost <= balance ifTrue: [planet iu: planet iu + amt]]].
		[$R] -> [cost := amt. cost <= balance ifTrue: [
					self point: 1@21; research: player field: key amt: amt; printRes: key]].
		[$V] -> [cost := amt. cost <= balance ifTrue: [
					self point: 1@21; research: player field: key amt: amt; printRes: key]].
		[$W] -> [cost := amt. cost <= balance ifTrue: [
					self point: 1@21; research: player field: key amt: amt; printRes: key]].
		[$ ] -> [].
		[$>] -> [
			self point: 1@18.
			self printf: '>?     '.
			self point: 3@18.
			self getChar caseOf: {
				[$M] -> [self printMap].
				[$S] -> [self starSum].
				[$C] -> [self printCol].
				[$R] -> [self resSum]
			} otherwise: [self errorMsg: ' !!Only M,S,C,R allowed      ']]
	} otherwise: [self errorMsg: ' !!Illegal field %c' with: key].
	cost > balance
		ifTrue: [self errorMsg: ' !!can''t affort %3d%c' with: amt with: key]
		ifFalse: [balance := balance - cost].
	key ~= $ ] whileTrue.
	self clearLeft.
	self point: 1@19.
	self printf: '%c' with: star name.
	self printf: '%d:%2d                         '
		with: planet number
		with: planet pSeeCapacity.
	self point: 6@19.
	self printf: '(%2d,/%3d)'
		with: planet inhabitants
		with: planet iu.
	self printf: (planet conquered ifTrue: ['Con'] ifFalse: ['   ']).
	planet mb > 0
		ifTrue: [self printf: '%2dmb' with: planet mb]
		ifFalse: [self printf: '    '].
	planet amb > 0
		ifTrue: [self printf: '%2damb' with: planet amb].
	self point: 1@20.
 	printtf ifTrue: [self printTf: tf].
	balance > 0] whileTrue.
	self zeroTf: tf team: player.
	self onBoard: star position
! !

!CqGame methodsFor: 'commands' stamp: 'sma 12/18/1999 22:29'!
land
	| tf star planet num roomLeft args transports cur |
	self printf: 'and tf:'.
	tf := player tfByName: self getChar.
	self clearLeft.
	tf isNil ifTrue: [^ self errorMsg: '  !!illegal tf'].
	tf exists ifFalse: [^ self errorMsg: '  !!nonexistent tf'].
	tf eta = 0 ifFalse: [^ self errorMsg: '  !!tf is not in normal space'].

	star := tf dest.
	star planets isEmpty ifTrue: [^ self errorMsg: '  !!no planets at this star'].
	(enemy tfsAtStar: star) isEmpty ifFalse: [^ self errorMsg: '  !!enemy ships present'].
	
	self point: 11@18.
	self printf: ' planet '.
	star planets size = 1 ifTrue: [
		planet := star planets first.
		self printf: '%d' with: planet number]
	ifFalse: [
		self printf: ':'.
		num := self getChar asInteger - 48.
		planet := star planets detect: [:p | p number = num]
			ifNone: [^ self errorMsg: '  !!not a habitable planet']].
	
	(planet team = enemy or: [planet team = player and: [planet conquered]])
		ifTrue: [^ self errorMsg: '  !!enemy infested planet'].

	roomLeft := planet capacity - planet inhabitants.
	self point: 1@19.
	self printf: ' transports:'.
	args := Array with: self getLine with: 0.
	self getToken: args.
	transports := args last.
	transports = 0 ifTrue: [transports := tf t].
	
	transports < 1 ifTrue: [^ self errorMsg: '  !!illegal transports'].
	transports > roomLeft ifTrue: [^ self errorMsg: '  !!only room for %d transports' with: roomLeft].

	planet team: player.
	"if (pplanet->inhabitants == 0)
    col_stars[starnum][player]++;"
	planet inhabitants: planet inhabitants + transports.
	planet iu: planet iu + transports.
	tf t: tf t - transports.

	(board at: tf position) enemy = $  ifTrue: [
		(board at: tf position) enemy: $@.
		self updateBoard: tf position on: #left].

	self point: 1@20.
	self printf: '%c' with: star name.

	((self cursor y > 21 and: [self cursor x >= 50]) or: [self cursor y > 24]) ifTrue: [
		self pause.
		self clearField.
		self point: 50@1].
	cur := self cursor.

	self printf: '%d:%2d                         ' 
		with: planet number
		with: planet pSeeCapacity.
	self point: cur + (5@0).
	self printf: '(%2d/%3d)   '
		with: planet inhabitants
		with: planet iu.
	planet mb > 0
		ifTrue: [self printf: '%2dmb' with: planet mb]
		ifFalse: [self printf: '    '].
	planet amb > 0
		ifTrue: [self printf: '%2damb' with: planet amb].
	self point: cur + (0@1).
	self zeroTf: tf team: player.
	self printTf: tf
! !

!CqGame methodsFor: 'commands' stamp: 'sma 12/11/1999 23:33'!
quit
	console clear; print: 'Quit game (y/n)?'.
	console nextChar asUppercase = $Y
		ifTrue: [gameOver := true]
		ifFalse: [self printMap]! !

!CqGame methodsFor: 'commands' stamp: 'sma 12/18/1999 22:37'!
sendTf
	| tf |
	self printf: 'estination tf:'.
	tf := player tfByName: self getChar.
	self clearLeft.
	self point: 1@19.
	tf isNil ifTrue: [^ self errorMsg: '  !!illegal tf'].
	tf exists ifFalse: [^ self errorMsg: '  !!nonexistent tf'].
	(tf eta ~= 0 and: [tf origeta ~= tf eta or: [tf withdrew]])
		ifTrue: [^ self errorMsg: '  !!tf is not in normal space'].
	tf isBlasting ifTrue: [^ self errorMsg: '  !!tf is blasting a planet'].
	
	tf withdrew: false.
	self setDes: tf! !


!CqGame methodsFor: 'display' stamp: 'sma 12/19/1999 10:28'!
clearField
	self point: 50 @ self cursor y.
	[self cursor y < 20] whileTrue: [
		self printf: self blankLine.
		self point: 50 @ (self cursor y + 1)]! !

!CqGame methodsFor: 'display' stamp: 'sma 12/11/1999 11:39'!
clearLeft
	19 to: 24 do: [:i | self point: 1@i; printf: self blankLine]! !

!CqGame methodsFor: 'display' stamp: 'sma 12/18/1999 14:18'!
clearScreen
	console clear! !

!CqGame methodsFor: 'display' stamp: 'sma 12/11/1999 12:56'!
dispTf: tf 
	tf t > 0
		ifTrue: [self printf: '%2dt' with: tf t]
		ifFalse: [self printf: '   '].
	tf s > 0
		ifTrue: [self printf: '%2ds' with: tf s]
		ifFalse: [self printf: '   '].
	tf c > 0
		ifTrue: [self printf: '%2dc' with: tf c]
		ifFalse: [self printf: '   '].
	tf b > 0
		ifTrue: [self printf: '%2db' with: tf b]
		ifFalse: [self printf: '   ']! !

!CqGame methodsFor: 'display' stamp: 'sma 12/19/1999 12:05'!
displayForces: enTf and: plTf odds: odds
	"Display forces in tf battle. Return TRUE if any battle must happen."

	| enForces plForces enOdds plOdds |
	enForces := plForces := 0.

	self zeroTf: enTf team: enemy.
	self zeroTf: plTf team: player.

	enTf exists ifTrue: [
		enForces := enemy weaponsResearch * enTf c * 6 + (enTf b * 40)].
	plTf exists ifTrue: [
		plForces := player weaponsResearch * plTf c * 6 + (plTf b * 40)].
 
	self point: 50@1.
	enTf exists
		ifTrue: [self printStar: enTf dest]
		ifFalse: [plTf exists
			ifTrue: [self printStar: plTf dest]].
	self clearField.

	((enForces = 0 and: [plForces = 0]) or: [enTf exists not or: [plTf exists not]])
		ifTrue: [^ false].

	enOdds := (plForces / (enForces + enTf t + (enTf s  * 2))) asFloat min: 14.0.
	enOdds := ((0.8 log * enOdds) exp * 100) asInteger.

	plOdds := (enForces / (plForces + plTf t + (plTf s  * 2))) asFloat min: 14.0.
	plOdds := ((0.8 log * plOdds) exp * 100) asInteger.

	self point: 1@19.
	self printf: 'enemy %5d' with: enForces.
	enForces > 0
		ifTrue: [self printf: '(weap %2d)' with: enemy weaponsResearch]
		ifFalse: [self printf: '         '].
	self printf: 'sur: %3d.0' with: enOdds.

	self point: 1@20.
	self printf: 'player%5d' with: plForces.
	plForces > 0
		ifTrue: [self printf: '(weap %2d)' with: player weaponsResearch]
		ifFalse: [self printf: '         '].
	self printf: 'sur: %3d.0' with: plOdds.

	odds at: 1 put: enOdds.
	odds at: 2 put: plOdds.

	^ true
! !

!CqGame methodsFor: 'display' stamp: 'sma 12/11/1999 23:32'!
errorMsg: fmtString
	self errorMsg: fmtString with: nil! !

!CqGame methodsFor: 'display' stamp: 'sma 12/11/1999 23:32'!
errorMsg: fmtString with: arg
	self errorMsg: fmtString with: arg with: nil! !

!CqGame methodsFor: 'display' stamp: 'sma 12/18/1999 22:15'!
errorMsg: fmtString with: arg1 with: arg2
	self point: 1@24; printf: fmtString with: arg1 with: arg2! !

!CqGame methodsFor: 'display' stamp: 'sma 12/18/1999 15:10'!
printCol
	self printf: 'olonies:'.
	self point: 50@1.
	
	stars do: [:star |
		star planets do: [:planet |
			planet team == player ifTrue: [
				console putChar: star name.
				((self cursor y > 21 and: [self cursor >= 50]) or: [self cursor y > 24]) ifTrue: [
					self pause; clearField; point: 50@1].
				self printf: '%d:%2d                         '
					with: planet number
					with: planet pSeeCapacity.
				self point: 55@ self cursor y.
				self printf: '(%2d,/%3d)'
					with: planet inhabitants
					with: planet iu.
				planet conquered
					ifTrue: [self printf: 'Con']
					ifFalse: [self printf: '   '].
				planet mb > 0
					ifTrue: [self printf: '%2dmb' with: planet mb]
					ifFalse: [self printf: '    '].
				planet amb > 0
					ifTrue: [self printf: '%2damb' with: planet amb].
				self point: 50@(self cursor y + 1)]]].
	self clearField.
	self clearLeft! !

!CqGame methodsFor: 'display' stamp: 'sma 12/11/1999 15:28'!
printMap
	| sector |
	self clearScreen.
	15 to: 1 by: -1 do: [:i1 |
		(i1 = 1 or: [i1 \\ 5 = 0])
			ifTrue: [self printf: '%2d|' with: i1]
			ifFalse: [self printf: '  |'].
		1 to: 15 do: [:i2 |
			sector := (board at: i2@i1).
			self printf: '%c' with: sector enemy.
			self printf: '%c' with: sector star.
			self printf: '%c' with: sector tf].
		self printf: Character cr].
	self printf: '   '.
	1 to: 15 do: [:i1 | self printf: '---'].
	self printf: Character cr.
	self printf: '   '.
	1 to: 15 do: [:i1 |
		(i1 = 1 or: [i1 \\ 5 = 0])
			ifTrue: [self printf: '%2d ' with: i1]
			ifFalse: [self printf: '   ']].
	self printf: Character cr.
	
	self point: 33@18; printf: 'Turn: %3d' with: turn.
	self point: 33@19; printf: 'Production yr: %d' with: productionYear! !

!CqGame methodsFor: 'display' stamp: 'sma 12/18/1999 15:58'!
printPlanet: planet see: see
	| cur |
	cur := self cursor.

	self printf: '%d:%2d                         '
		with: planet number
		with: planet pSeeCapacity.
	self point: cur + (5@0).
	planet pSeeCapacity = 0 ifTrue: [self printf: ' Decimated']
	ifFalse: [
		planet team caseOf: {
			[nil] -> [see ifTrue: [self printf: ' No colony']].
			[player] -> [
				self printf: '(%2d,/%3d)'
					with: planet inhabitants
					with: planet iu.
				planet conquered
					ifTrue: [self printf: 'Con']
					ifFalse: [self printf: '   '].
				planet mb > 0
					ifTrue: [self printf: '%2dmb' with: planet mb]
					ifFalse: [self printf: '    '].
				planet amb > 0
					ifTrue: [self printf: '%2damb' with: planet amb]].
			[enemy] -> [
				see ifTrue: [
					self printf: '*EN*'.
					planet conquered
						ifTrue: [self printf: 'Conquered']
						ifFalse: [self printf: '   '].
					planet underAttack ifTrue: [
						planet mb > 0
							ifTrue: [self printf: '%2dmb' with: planet mb]
							ifFalse: [self printf: '    '].
						planet amb > 0
							ifTrue: [self printf: '%2damb' with: planet amb]]]]
		}].
		self point: cur + (-1@1)! !

!CqGame methodsFor: 'display' stamp: 'sma 12/18/1999 14:42'!
printStar: star
	| see |
	star isNil ifTrue: [^ self].

	self cursor y + 3 > 19 ifTrue: [
	"if ((y_cursor+3+tf_stars[stnum][player]+tf_stars[stnum][ENEMY]) > 19) "
		self clearField.
		self pause.
		self point: 50@1].

	(star visitedBy: player) ifTrue: [
		see := false.
		self printf: '----- star %c -----            ' with: star name.
		self point: 50@(self cursor y + 1).

		"Print taskforces"
		(player tfsAtStar: star) do: [:tf |
			see := true.
			self printf: 'TF%c                           ' with: tf name.
			self point: 55@self cursor y.
			self dispTf: tf.
			self point: 50@(self cursor y + 1)].
		
		see := see | ((player coloniesAtStar: star) > 0).
		
		see ifTrue: [
			(enemy tfsAtStar: star) do: [:tf |
				self printf: ' EN:                          '.
				self point: 55@self cursor y.
				self dispTf: tf.
				self point: 50@(self cursor y + 1)]].

		star planets isEmpty ifTrue: [
			self printf: '  no useable planets          '.
			self point: 50@(self cursor y + 1).
			^ self].

		star planets do: [:p |
			self printf: ' '.
			((self cursor y > 21 and: [self cursor x >= 50]) or: [self cursor y > 24]) ifTrue: [
				self pause.
				self clearField.
				self point: 50@1].
			self printPlanet: p see: see]]
! !

!CqGame methodsFor: 'display' stamp: 'sma 12/18/1999 22:36'!
printTf: tf
	| cur |
	(tf notNil and: [tf exists]) ifFalse: [^ self].
	cur := self cursor.
	self printf: 'TF%c:' with: tf name.
	tf eta = 0
		ifTrue: [self printf: '%c' with: tf dest name]
		ifFalse: [self printf: ' '].
	self printf: '(%2d,%2d)               '
		with: tf position x
		with: tf position y.
	self move: -14@0.
	self dispTf: tf.
	tf eta = 0 ifFalse: [
		self printf: '%c%d'
			with: tf dest name
			with: tf eta].
	self point: cur + (0@1)
! !

!CqGame methodsFor: 'display' stamp: 'sma 12/11/1999 12:13'!
starSum
	| args token |
	self printf: 'tar summary:'.
	self clearLeft.
	self point: 1@19.
	self printf: ':'.
	
	args := Array with: self getLine with: 0.
	token := self getToken: args.

	self point: 50@1.
	token = $  ifTrue: [stars do: [:star | self printStar: star]]
	ifFalse: [
		[self printStar: (self getStarByName: token).
		token := self getToken: args. token = $ ] whileFalse].
	self clearField! !

!CqGame methodsFor: 'display' stamp: 'sma 12/12/1999 00:33'!
tfSum
	| args token |
	self printf: 'f summary :'.
	
	args := Array with: self getLine with: 0.
	token := self getToken: args.

	self point: 50@1.
	
	token = $  ifTrue: [
		player tfs do: [:tf | self printTf: tf]]
	ifFalse: [
		[self printTf: (player tfByName: token).
		token := self getToken: args.
		token = $ ] whileFalse].

	self clearField.
	self clearLeft
! !


!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 20:56'!
bestStar: slist
	slist with: stars do: [:dist :star | dist > 0 ifTrue: [^ star]].
	^ nil! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 20:20'!
bestWithdrawPlan: star odds: enOdds
	"Return TRUE if we should all flee from this battle."

	| team size |
	team := nil.
	size := 0.
	star planets do: [:planet |
		planet capacity > size
			ifTrue:
				[size := planet capacity.
				team := planet team]].
	^ enOdds < 70 & (size < 30) or: [
		enOdds < 50 & (team = player) or: [
		enOdds < 30 & (size < 60) or: [
		enOdds < 20]]]! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 20:35'!
depart: star
	(player tfsAtStar: star) size + (player coloniesAtStar: star) > 0
		ifTrue: [enemy departures add: star]! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/19/1999 14:49'!
enemyAttack: star
	| first enTf attFactors bestPlanet bestScore defFactors odds |
	"Enemy attack at a planet"

	first := Array new: 6 withAll: true.
	enTf := enemy tfs detect: [:tf | tf dest = star and: [tf eta = 0]].
	
	[attFactors := enTf b * 6 + enTf c.
	bestPlanet := nil.
	bestScore := 1000.

	"Calculate the best one to attack"
	star planets do: [:planet |
		planet team = player ifTrue: [
			defFactors := planet eSeeDef.
			odds := (defFactors/ attFactors) asFloat.
			odds := planet capacity > 30
				ifTrue: [odds - 2 * planet capacity]
				ifFalse: [odds - 1.5 * planet capacity].
			odds < bestScore ifTrue: [bestScore := odds. bestPlanet := planet]]].

	"Found one that was worth attacking"
	bestScore < 0 ifTrue: [
		self clearLeft.
		self point: 1@19.
		self printf: 'Enemy attacks: %c%d' with: star name with: bestPlanet number.
		self point: 50@1.
		self printStar: star.
		self clearField.
		self pause.

		self fireSalvo: enemy tf: enTf planet: bestPlanet firstTime: (first at: bestPlanet number).
		first at: bestPlanet number put: false.
		self zeroTf: enTf team: enemy.
		bestPlanet eSeeDef: bestPlanet amb * 6 + bestPlanet mb.
		
		self pause].
	bestScore < 0 and: [enemy anyBc: star]] whileTrue.

	self revolt: star! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/18/1999 20:23'!
evalBcCol: planet ofStar: star
	"How much do we care about moving ships to this planet"

	| result |
	(star visitedBy: enemy)
		ifFalse: [result := 600]
		ifTrue:
			[planet eSeeTeam caseOf: {
				[nil] -> [result := 100].
				[enemy] -> [planet conquered
					ifTrue: [result := 1000]
					ifFalse: [(planet amb * 6 + planet mb <= (planet iu // 15)
						and: [planet iu >= 8]) ifTrue: [result := 300] ifFalse: [result := 0]].
					planet amb >= 4 ifTrue: [result := result - 250]].
				[player] -> [planet conquered
					ifTrue: [result := 400]
					ifFalse: [result := 200]]
			}.
			(planet capacity < 40 and: [planet iu < 15])
				ifTrue: [result := result - 100]].
	^ result + (CqGame rnd: 20)! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 20:47'!
evalTCol: planet range: range ofStar: star
	| result |
	(star visitedBy: enemy)
		ifFalse: [result := 60]
		ifTrue: [
			planet eSeeTeam caseOf: {
				[nil] -> [result := 40].
				[enemy] -> [result := 30].
				[player] -> [result := 0]
			}.
			(planet eSeeTeam ~= player
				and: [planet capacity - planet inhabitants > (40 - (turn // 2))])
				ifTrue: [result := result + 40]].
	^ result - ((range * 2) + 0.5) rounded! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/18/1999 22:27'!
inputMachine
	| slist |
	slist := Array new: stars size.

	enemy tfs do: [:tf |
		(tf exists and: [tf eta = 0]) ifTrue: [
			self getStars: tf dest into: slist.
			self sendScouts: slist tf: tf.
			self sendTransports: slist tf: tf.
			self moveBc: slist tf: tf.
			self zeroTf: tf team: enemy]]
! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/18/1999 20:24'!
investEnemy: star planet: planet
	| balance newTf minMb blocks amt |
	balance := planet iu.
	(enemy tfsAtStar: star) size = 0
		ifTrue: [newTf := enemy getTf: star]
		ifFalse: [newTf := (enemy tfsAtStar: star) first].

	"Build a minimum of missile bases"
	minMb := planet capacity // 20.
	[planet amb = 0 and: [planet conquered not and: [planet mb < minMb and: [balance >= 8]]]]
		whileTrue:
			[balance := balance - 8.
			planet mb: planet mb + 1].

	"Build an amb more"
	(balance >= 70 and: [planet amb > 1 and: [(CqGame rnd: 5) ~= 1 and: [(CqGame rnd: 7) - 3 <= planet amb]]])
		ifTrue:
			[balance := balance - 70.
			newTf b: newTf b + 1].

	"Maybe build the first one"
	(balance >= 35 and: [planet conquered not and: [planet amb < 4 and: [(CqGame rnd: 2) = 2]]])
		ifTrue:
			[balance := balance - 35.
			planet amb: planet amb + 1].
	blocks := {
		[self research: enemy field: enResearch amt: 8. balance := balance - 8].
		[balance >= 16
			ifTrue: [balance := balance - 16. newTf c: newTf c + 1]
			ifFalse: [(planet conquered not and: [balance >= 8])
				ifTrue: [balance := balance - 8. planet mb: planet mb + 1]
				ifFalse: [(blocks at: 1) value]]].
		[(planet inhabitants / planet capacity < 0.6
			or: [planet capacity >= 35 and: [planet iu < 80]])
			ifTrue: [amt := 3 min: planet inhabitants * 3 - planet iu.
				balance := balance - (amt * 3).
				planet iu: planet iu + amt]
			ifFalse: [planet conquered ifFalse: [
				amt := 6 + (CqGame rnd: 3) min: planet inhabitants - 1.
				planet iu > 70 ifTrue: [amt := amt min: planet iu - 70].
				balance := balance - amt.
				planet inhabitants: planet inhabitants - amt.
				planet iu: (planet iu - amt min: planet inhabitants * 2).
				newTf t: newTf t + amt]]]
	}.
	[balance >= 9]
		whileTrue:
			[(CqGame rnd: 12) caseOf: {
			[1] ->	[(blocks at: 1) value].
			[2] ->	[(blocks at: 1) value].
			[3] ->	[(blocks at: 2) value].
			[4] ->	[(blocks at: 2) value].
			[10] ->	[(blocks at: 2) value].
			[11] ->	[(blocks at: 3) value].
			[12] ->	[(blocks at: 3) value]
			} otherwise: [
				amt := 3 min: planet inhabitants * 2 - planet iu.
				balance := balance - (amt * 3).
				planet iu: planet iu + amt]].
	self zeroTf: newTf team: enemy.
	self research: enemy field: enResearch amt: balance! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/18/1999 20:25'!
moveBc: slist tf: tf
	| bestStar bestPlanet topScore score factors |
	tf b + tf c = 0 ifTrue: [^ self].

	bestStar := self bestStar: slist.
	bestPlanet := nil.
	topScore := -1000.

	slist with: stars do: [:dist :star |
		(dist > 0 or: [tf dest = star]) ifTrue: [
			star planets do: [:planet |
				score := self evalBcCol: planet ofStar: star.
				star = tf dest ifTrue: [score := score + 250].
				(enemy tfsAtStar: star) size > 0 ifTrue: [score := score - 150].
				
				score > topScore ifTrue: [
					topScore := score.
					bestPlanet := planet.
					bestStar := star]]]].
	bestStar = tf dest ifTrue: [
		"stay put"
		(bestPlanet notNil and: [bestPlanet team = enemy and: [bestPlanet conquered]]) ifTrue: [
			bestPlanet iu < 20 ifTrue: [
				"blast it"
				factors := enemy weaponsResearch * ((tf c * 7) + (tf b * 40)).
				factors := factors min: bestPlanet inhabitants * 4.
				self blast: bestPlanet with: factors.
				(player tfsAtStar: bestStar) size > 0 ifTrue: [
					bestPlanet pSeeCapacity: bestPlanet capacity]]
			ifFalse: [
				"Decide whether to split"
				(((tf b > 3 or: [tf c > 3]) and: [(CqGame rnd: 4) = 4]) or: [tf b > 8])
					ifTrue: [self wanderBc: slist tf: tf]]]]
	ifFalse: [
		"move"
		self sendTf: tf to: bestStar slist: slist]! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/19/1999 11:10'!
sendScouts: slist tf: tf
	| doable newTf dest i |
	tf s > 0 ifFalse: [^ self].

	doable := OrderedCollection new: tf s.
	slist with: stars do: [:dist :star |
		(dist > 0 and: [(star visitedBy: enemy) not]) ifTrue: [
			doable add: star]].

	[doable isEmpty not and: [tf s > 0]] whileTrue: [
		newTf := enemy getTf: tf dest.
		newTf s: 1.
		tf s: tf s - 1.
		dest := doable at: (CqGame rnd: doable size).
		doable remove: dest.
		
		self sendTf: newTf to: dest slist: slist].

	[tf s > 0] whileTrue: [
		[(slist at: (i := CqGame rnd: stars size)) <= 0] whileTrue.
		newTf := enemy getTf: tf dest.
		newTf s: 1.
		tf s: tf s - 1.
		self sendTf: newTf to: (stars at: i) slist: slist]! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 21:11'!
sendTf: tf to: star slist: slist
	self depart: tf dest.
	tf dest: star.
	tf eta: (((slist at: (stars indexOf: star)) - 0.01) / enemy velocityResearch) asInteger + 1! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 21:28'!
sendTransports: slist tf: tf
	| bestStar secStar topScore secScore bestPlanet score toLand newTf |
	tf t > 0 ifFalse: [^ self].

	bestStar := nil.
	secStar := nil.
	topScore := -11000.
	secScore := -11000.
	bestPlanet := nil.

	slist with: stars do: [:dist :star |
		(dist > 0 or: [star = tf dest]) ifTrue: [
			star planets do: [:planet |
				score := self evalTCol: planet range: dist ofStar: star.
				score > secScore ifTrue: [
					score > topScore ifTrue: [
						secScore := topScore.
						secStar := bestStar.
						topScore := score.
						bestStar := star.
						bestPlanet := planet]
					ifFalse: [
						secScore := score.
						secStar := star]]]]].
	bestStar = tf dest ifTrue: [
		"Land"
		((player tfsAtStar: bestStar) size = 0 and: [bestPlanet team ~= player]) ifTrue: [
			toLand := (bestPlanet capacity - bestPlanet inhabitants) // 3 min: tf t.
			toLand > 0 ifTrue: [
				bestPlanet inhabitants = 0 ifTrue: [
					bestPlanet team: enemy.
					bestPlanet eSeeTeam: enemy].
				bestPlanet inhabitants: bestPlanet inhabitants + toLand.
				bestPlanet iu: bestPlanet iu + toLand.
				tf t: tf t - toLand.
				self sendTransports: slist tf: tf]]]
	ifFalse: [
		"Move"
		(tf t >= 10 and: [secStar notNil]) ifTrue: [
			"First send half to the best star, if two stars"
			newTf := enemy getTf: tf dest.
			newTf t: tf t // 2.
			tf t: tf t - newTf t.
			(tf c > 0 and: [(self underdefended: tf dest) not]) ifTrue: [
				newTf c: 1.
				tf c: tf c - 1].
			self sendTf: newTf to: bestStar slist: slist.
			bestStar := secStar].
		newTf := enemy getTf: tf dest.
		newTf t: tf t.
		tf t: 0.
		(tf c > 0 and: [(self underdefended: tf dest) not]) ifTrue: [
			newTf c: 1.
			tf c: tf c - 1].
		self sendTf: newTf to: bestStar slist: slist]! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/16/1999 21:47'!
underdefended: star
	star planets do: [:planet |
		(planet team = enemy 
			and: [planet iu > 10 
			and: [planet amb * 6 + planet mb < (planet iu / 15)]]) ifTrue: [^ true]].
	^ false! !

!CqGame methodsFor: 'enemy' stamp: 'sma 12/18/1999 20:26'!
wanderBc: slist tf: tf
	| reachable dest newTf |
	(tf b > 1 or: [tf c > 1]) ifFalse: [^ self].
	
	reachable := OrderedCollection new.
	slist with: stars do: [:dist :star | dist > 0 ifTrue: [reachable add: star]].
	dest := reachable at: (CqGame rnd: reachable size).
	newTf := enemy getTf: dest.
	newTf b: tf b // 2.
	newTf c: tf c // 2.
	tf b: tf b - newTf b.
	tf c: tf c - newTf c.
	tf t > 3 ifTrue: [
		newTf t: 2.
		tf t: tf t - 2].
	self sendTf: newTf to: dest slist: slist! !


!CqGame methodsFor: 'init' stamp: 'sma 12/11/1999 19:21'!
initBoard
	board := CqBoard new.
	stars := board stars! !

!CqGame methodsFor: 'init' stamp: 'sma 12/18/1999 22:13'!
initGame
	gameOver := false.
	turn := 1.
	productionYear := 1.

	player := CqPlayer new.
	enemy := CqPlayer new! !

!CqGame methodsFor: 'init' stamp: 'sma 12/18/1999 20:24'!
initMachine
	| tf maxx slist count startStar |

	enemy rangeResearch: 7; growthRate: 0.5.
	
	"Small cheats for the enemy"
	(CqGame rnd: 3) caseOf: {
		[1] ->	[enemy weaponsResearch: (CqGame rnd: 4) + 2].
		[2] ->	[enemy velocityResearch: (CqGame rnd: 3)].
		[3] ->	[enemy growthRate: (CqGame rnd: 4) + 3 / 10.0]
	}.

	"Enemy spends 30 units on 1c, 2s and 2v."
	enResearch := $V.
	self research: enemy field: enResearch amt: 2.

	"Find out the best place to start (most stars reachable)"
	maxx := 0.
	stars do: [:star |
		slist := Array new: stars size.
		count := self getStars: star into: slist.
		count := count + (CqGame rnd: 5).
		count > maxx ifTrue: [maxx := count. startStar := star]].

	tf := enemy tfByName: $a.
	tf t: 35.
	tf c: 1.
	tf s: 2.
	tf dest: startStar.
	tf position: startStar position.
	startStar beVisitedBy: enemy
! !

!CqGame methodsFor: 'init' stamp: 'sma 12/19/1999 10:24'!
initPlayer
	| initunit initmoney star tf balance args cost key amt |
	initunit := 30.
	initmoney := 35.

	self printMap.
	self point: 33@20.
	self printf: '*Initialization*'.
	
	[self point: 1@18.
	self printf: 'start at star? '.
	star := self getStarByName: self getChar.
	star isNil] whileTrue.

	tf := player tfByName: $a.
	tf position: star position; dest: star.
	tf t: initunit.

	self point: 1@20.
	self printf: 'choose your initial fleet.'.
	self point: 1@21.
	self printf: 'you have %d transports' with: initunit.
	self point: 1@22.
	self printf: 'and %d units to spend' with: initmoney.
	self point: 1@23.
	self printf: 'on ships or research.'.

	self printRes: $V.
	self printRes: $R.
	self printRes: $W.

	balance := initmoney.
	[self point: 1@19.
	self printTf: tf.
	self point: 1@18.
	self printf: '%3d?                          ' with: balance.
	self point: 6@18.
	args := Array with: self getLine with: 0.
	[cost := 0.
	key := self getToken: args.
	amt := args last.
	key caseOf: {
		[$T] ->	[cost := amt * 1. cost <= balance ifTrue: [tf t: tf t + amt]].
		[$S] ->	[cost := amt * 6. cost <= balance ifTrue: [tf s: tf s + amt]].
		[$C] ->	[cost := amt * 16. cost <= balance ifTrue: [tf c: tf c + amt]].
		[$B] ->	[cost := amt * 70. cost <= balance ifTrue: [tf b: tf b + amt]].
		[$H] ->	[self help: 0].
		[$W] ->	[cost := amt. cost <= balance ifTrue: [
					self research: player field: key amt: amt. self printRes: key]].
		[$V] ->	[cost := amt. cost <= balance ifTrue: [
					self research: player field: key amt: amt. self printRes: key]].
		[$R] ->	[cost := amt. cost <= balance ifTrue: [
					self research: player field: key amt: amt. self printRes: key]].
		[$ ] ->	[].
		[$>] ->	[self point: 1@18.
				self printf: '>?      '.
				self point: 3@18.
				key := self getChar.
				key caseOf: {
					[$M] ->	[self printMap].
					[$R] ->	[self resSum]
				} otherwise: [self errorMsg: ' !!Only M,R during initialize']]
	} otherwise: [self errorMsg: ' !!Illegal field %c' with: key].

	cost <= balance
		ifTrue: [balance := balance - cost]
		ifFalse: [self errorMsg: '  !!can''t afford %d%c' with: amt with: key].
	key = $ ] whileFalse.
	balance > 0] whileTrue.
	
 	star beVisitedBy: player.
	(board at: star position) tf: $a; enemy: $ .
	self onBoard: star position.
	self point: 33@20
! !

!CqGame methodsFor: 'init' stamp: 'sma 4/29/2000 10:26'!
startup
	console clear; cr;
		print: '* Welcome to CONQUEST!! *'; cr; cr;
		print: 'Smalltalk version 1.0'; cr;
		print: 'Hit return to continue';
		nextChar.

	self showInstructions.
	
	self initGame.
	self initBoard.
	self initPlayer.
	self initMachine.

	"Put player on board"
	self point: 50@1.
	self printStar: (player tfByName: $a) dest.
	self clearField.

	"And fight if in same place"
	(enemy tfsAtStar: (player tfByName: $a) dest) size > 0
		ifTrue:
			[self clearLeft.
			self battle]! !


!CqGame methodsFor: 'input' stamp: 'sma 12/11/1999 15:29'!
getChar
	"Read in a single char, echo it on screen, and answer it."

	| ch |
	ch _ console nextChar asUppercase.
	ch = Character lf ifTrue: [ch _ Character cr].
	console putChar: ch.
	^ ch! !

!CqGame methodsFor: 'input' stamp: 'sma 12/11/1999 13:56'!
getLine
	"Read in a CR-terminated line (up to 25 characters) and answer it."

	| line ch |
	line _ WriteStream on: (String new: 25).
	[ch _ self getChar.
	ch = Character backspace
		ifTrue: [line position > 0
				ifTrue: 
					[console
						putChar: Character space;
						putChar: Character backspace.
					line skip: -1]
				ifFalse: [console move: 1 @ 0]]
		ifFalse: [ch = Character cr ifFalse: [line nextPut: ch]].
	line position < 25 and: [ch ~= Character cr]] whileTrue.
	^ line contents! !

!CqGame methodsFor: 'input' stamp: 'sma 12/16/1999 19:47'!
getStars: star into: slist
	"Compile a list of distances."

	| count |
	count := 0.
	1 to: stars size do: [:i |
		(slist at: i put: (star dist: (stars at: i))) <= enemy rangeResearch
			ifTrue: [count := count + 1]
			ifFalse: [slist at: i put: 0]].
	^ count
! !

!CqGame methodsFor: 'input' stamp: 'sma 12/5/1999 18:03'!
getToken: args 
	"Grab character and optionally preceeding number from input argument.
	Answer the character, called token here, and adjust the input line. The
	number is passed by reference using the last element of 'args'."

	| value token input |
	value _ 0.
	token _ $ .
	input _ ReadStream on: args first.
	input skipSeparators.
	input atEnd
		ifFalse: 
			[input peek isDigit
				ifTrue: [[input atEnd not and: [input peek isDigit]]
						whileTrue: [value _ value * 10 + input next digitValue]]
				ifFalse: [value _ 1].
			input atEnd ifFalse: [token _ input next]].
	args at: 1 put: input upToEnd.
	args at: 2 put: value.
	^ token! !


!CqGame methodsFor: 'main' stamp: 'sma 12/16/1999 20:08'!
runOnConsole: aConsole
	console := aConsole.

	self startup.
	[self inputPlayer.
	gameOver
		ifFalse:
			[self inputMachine.
			self moveShips.
			self battle.
			self production.
			self nextYear].
	self checkGameOver.
	gameOver] whileFalse! !


!CqGame methodsFor: 'misc' stamp: 'sma 12/11/1999 23:03'!
help: which
	| h i |
	h := self helpTexts at: which + 1.
	i := 0.
	1 to: h size by: 2 do: [:j |
		self point: 50@(i := i + 1).
		self printf: '%2s - %-25s' with: (h at: j) with: (h at: j + 1)].
	self clearField! !

!CqGame methodsFor: 'misc' stamp: 'sma 12/18/1999 22:53'!
onBoard: position
	| sector |
	sector := board at: position.
	sector tf: $ .
	player tfs do: [:tf |
		(tf exists and: [tf position = position]) ifTrue: [
			sector tf = $ 
				ifTrue: [sector tf: tf name]
				ifFalse: [sector tf: $*]]].
	sector star = $. ifFalse: [
		"check whether this is a world populated by the enemy..."
		(enemy coloniesAtStar: (self getStarByName: sector star)) > 0
			ifTrue: [sector enemy: $@]
			ifFalse: [sector enemy: $ ]].
	self updateBoard: position on: #both! !

!CqGame methodsFor: 'misc' stamp: 'sma 12/18/1999 22:18'!
pause
	self point: 1 @ 18.
	self printf: 'Press any key to continue     '.
	self getChar! !


!CqGame methodsFor: 'movement' stamp: 'sma 12/18/1999 22:27'!
fleetExplore: tf
	| prob loss |
	self clearLeft.
	self point: 1@19.
	self printf: 'Task force %c exploring %c.'
		with: tf name
		with: tf dest name.
	self point: 1@20.

	prob := 0.
	tf t > 0 ifTrue: [prob := 10 + ((CqGame rnd: 5) * tf t)].
	tf s > 0 ifTrue: [prob := 70 + ((CqGame rnd: 10) * tf s)].
	tf c > 0 ifTrue: [prob := 90 + ((CqGame rnd: 10) * tf c)].
	tf b > 0 ifTrue: [prob := 95 + ((CqGame rnd: 10) * tf b)].
	prob > 100 ifTrue: [prob := 100].

	loss := false.
	loss := loss | (self lose: (CqRef object: tf aspect: #t) ofType: $t chance: prob).
	loss := loss | (self lose: (CqRef object: tf aspect: #s) ofType: $s chance: prob).
	loss := loss | (self lose: (CqRef object: tf aspect: #c) ofType: $c chance: prob).
	loss := loss | (self lose: (CqRef object: tf aspect: #b) ofType: $b chance: prob).

	loss ifFalse: [self printf: 'No ships'].
	self printf: ' destroyed.'.
	
	self zeroTf: tf team: player.
	tf exists ifTrue: [
		"Anything left?"
		(board at: tf position) enemy: $ .
		self updateBoard: tf position on: #left.
		tf dest beVisitedBy: player.
		self point: 50@1.
		self printStar: tf dest.
		self clearField].
	self pause
! !

!CqGame methodsFor: 'movement' stamp: 'sma 4/29/2000 11:01'!
lose: ships ofType: type chance: percent 
	| anyLosses shipsLeft |
	anyLosses := false.
	shipsLeft := ships value.
	1 to: ships value do: [:i |
		(CqGame rnd: 100) >= percent
			ifTrue: 
				[anyLosses := true.
				shipsLeft := shipsLeft - 1]].
	anyLosses
		ifTrue: 
			[self printf: '%2d%c' with: ships value - shipsLeft with: type.
			ships value: shipsLeft].
	^ anyLosses! !

!CqGame methodsFor: 'movement' stamp: 'sma 12/18/1999 22:30'!
moveShips
	| ratio there any |
	"clear the board"
	player tfs do: [:tf |
		(tf exists and: [tf eta > 0]) ifTrue: [
			(board at: tf position) tf: $ .
			self updateBoard: tf position on: #right]].

	"move ships of both teams"
	(Array with: enemy with: player) do: [:tm |
		tm tfs do: [:tf |
			(tf exists and: [tf eta > 0]) ifTrue: [
				tf eta: tf eta - 1.
				tm = player ifTrue: [
					ratio := 1.0 - (tf eta / tf origeta) asFloat.
					tf position: (tf dest position - tf positionf * ratio) rounded + tf positionf.
					tf eta = 0 ifTrue: [
						"Update for arrival"
						(tf dest visitedBy: tm) ifFalse: [self fleetExplore: tf].
						tf dest notNil ifTrue: [
							"Any survivors?"
							tf dest planets do: [:p | p pSeeCapacity: p capacity].
							tm arrivals add: tf dest]]].
				(tm = enemy and: [tf eta = 0]) ifTrue: [
					tf dest beVisitedBy: tm.
					tf dest planets do: [:p | p eSeeTeam: p team].
					(there := (enemy tfsAtStar: tf dest) reject: [:t | t == tf]) size > 0 ifTrue: [
						self joinsilent: tf with: there first].
					((player coloniesAtStar: tf dest) > 0 or: [(player tfsAtStar: tf dest) size > 0]) ifTrue: [
						tm arrivals add: tf dest]]]]].

	"put the good guys on the board"
	player tfs do: [:tf |
		tf exists ifTrue: [
			tf isBlasting: false.
			(board at: tf position) tf = $ 
				ifTrue: [(board at: tf position) tf: tf name]
				ifFalse: [(board at: tf position) tf ~= tf name
					ifTrue: [(board at: tf position) tf: $*]].
			self updateBoard: tf position on: #right]].

	any := false.
	player arrivals do: [:star |
		any ifFalse: [
			self point: 33@21.
			self printf: 'Player arrivals :               '.
			self point: 50@21.
			any := true].
		console putChar: star name].
	any ifFalse: [self point: 33@21; printf: self blankLine].

	any := false.
	enemy arrivals do: [:star |
		any ifFalse: [
			self point: 33@22.
			self printf: 'Enemy arrivals  :               '.
			self point: 50@22.
			any := true].
		console putChar: star name].
	any ifFalse: [self point: 33@22; printf: self blankLine].

	any := false.
	enemy departures do: [:star |
		any ifFalse: [
			self point: 33@23.
			self printf: 'Enemy departures:               '.
			self point: 50@23.
			any := true].
		console putChar: star name].
	any ifFalse: [self point: 33@23; printf: self blankLine].

	player resetArrivals.
	enemy resetArrivals; resetDepartures.

	stars do: [:star | self revolt: star]! !

!CqGame methodsFor: 'movement' stamp: 'sma 12/19/1999 10:34'!
setDes: tf
	| star r dst minEta |
	tf eta ~= 0 ifTrue: [
		"Cancelling previous orders"
		tf eta: 0.
		tf dest: (self getStarByName: (board at: tf position) star).
		self printf: '(Cancelling previous orders)'.
		self point: 1 @ (self cursor y + 1)].
	self printf: ' to star: '.
	star := self getStarByName: self getChar.
	star isNil ifTrue: [self errorMsg: '  !!illegal star'.  ^ true].
 
	r := star dist: tf dest.
	
	self point:  1@ (self cursor y + 1).
	self printf: '   distance: %d' with: (r roundTo: 0.1).

	dst := (r - 0.049) truncated + 1.
	
	(dst > player rangeResearch and: [tf b > 0 or: [tf c > 0 or: [tf t > 0]]])
		ifTrue: [self errorMsg: '  !!maximum range is %d' with: player rangeResearch. ^ true].

	r < 0.5 ifTrue: [
		self point: 1 @ (self cursor y + 1).
		self printf: 'Tf remains at star'.
		^ true].

	minEta := dst - 1 // player velocityResearch + 1.
	self point: 1 @ (self cursor y + 1).
	self printf: 'eta in %2d turns' with: minEta.
	
	tf dest: star.
	tf eta: minEta.
	tf origeta: minEta.
	tf positionf: tf position.
	^ false! !


!CqGame methodsFor: 'research' stamp: 'sma 12/18/1999 20:25'!
newResearch
	"Private. Enemy picks new research field based on player stats."

	enemy weaponsResearch < enemy weaponsReq size ifTrue: [
		player weaponsResearch - enemy weaponsResearch + 5 > (CqGame rnd: 10) ifTrue: [
			enResearch = $W.
			^ self]].
	enemy velocityResearch < enemy velocityReq size ifTrue: [
		enemy rangeResearch / enemy velocityResearch > (CqGame rnd: 3) ifTrue: [
			enResearch = $V.
			^ self]].
	enResearch = $R! !

!CqGame methodsFor: 'research' stamp: 'sma 12/19/1999 10:20'!
printRes: field
	"Display a research sumary for the specified field."

	field caseOf: {
		[$V] -> [
			self point: 53@18.
			self printf: 'V:%2d' with: player velocityResearch.
			player velocityResearch < player velocityReq size ifTrue: [
				self printf: ' res: %3d need:%4d'
					with: player velocityWorking
					with: (player velocityReq at: player velocityResearch + 1)]
			ifFalse: [self printf: '                   ']].
		[$R] -> [
			self point: 53@19.
			self printf: 'R:%2d' with: player rangeResearch.
			player rangeResearch < player rangeReq size ifTrue: [
				self printf: ' res: %3d need:%4d'
					with: player rangeWorking
					with: (player rangeReq at: player rangeResearch + 1)]
			ifFalse: [self printf: '                   ']].
		[$W] -> [
			self point: 53@20.
			self printf: 'W:%2d' with: player weaponsResearch.
			player weaponsResearch < player weaponsReq size ifTrue: [
				self printf: ' res: %3d need:%4d'
					with: player weaponsWorking
					with: (player weaponsReq at: player weaponsResearch + 1)]
			ifFalse: [self printf: '                   ']]
	} otherwise: []! !

!CqGame methodsFor: 'research' stamp: 'sma 12/12/1999 22:14'!
resSum
	"Display a research summary for one or more fields."

	| args key |
	self printf: 'esearch field(s):'.
	args := Array with: self getLine with: 0.
	key := self getToken: args.
	key = $ 
		ifTrue: [
			self printRes: $R.
			self printRes: $V.
			self printRes: $W]
		ifFalse: [
			[self printRes: key.
			key := self getToken: args.
			key = $ ] whileFalse]! !

!CqGame methodsFor: 'research' stamp: 'sma 12/18/1999 22:16'!
research: team field: field amt: amt
	"Spend points on research for a given field."

	| amt0 field0 oldAmt |
	amt0 := amt.
	field0 := field.
	[oldAmt := amt0.
	amt0 := amt0 - (self researchLimited: team field: field0 amt: amt0).
	team = enemy ifTrue: [
		self newResearch.
		field0 := enResearch].
	oldAmt = amt0 ifTrue: [
		"How stupid - invested beyond the limit"
		amt0 := 0].
	amt0 > 0] whileTrue! !

!CqGame methodsFor: 'research' stamp: 'sma 12/12/1999 22:16'!
researchLimited: team field: field amt: maxAmt
	"Private. Do some research up to the next level and return amount needed."

	| used |
	used := 0.
	field caseOf: {
		[$W] -> [
			team weaponsResearch < team weaponsReq size ifTrue: [
				used := maxAmt min: (team weaponsReq at: team weaponsResearch + 1) - team weaponsWorking.
				team weaponsWorking: team weaponsWorking + used.
				team weaponsWorking >= (team weaponsReq at: team weaponsResearch + 1) ifTrue: [
					team weaponsResearch: team weaponsResearch + 1.
					team = enemy ifTrue: [self newResearch].
					team weaponsWorking: 0]]].
		[$R] -> [
			team rangeResearch < team rangeReq size ifTrue: [
				used := maxAmt min: (team rangeReq at: team rangeResearch + 1) - team rangeWorking.
				team rangeWorking: team rangeWorking + used.
				team rangeWorking >= (team rangeReq at: team rangeResearch + 1) ifTrue: [
					team rangeResearch: team rangeResearch + 1.
					team = enemy ifTrue: [self newResearch].
					team rangeWorking: 0]]].
		[$V] -> [
			team velocityResearch < team velocityReq size ifTrue: [
				used := maxAmt min: (team velocityReq at: team velocityResearch + 1) - team velocityWorking.
				team velocityWorking: team velocityWorking + used.
				team velocityWorking >= (team velocityReq at: team velocityResearch + 1) ifTrue: [
					team velocityResearch: team velocityResearch + 1.
					team = enemy ifTrue: [self newResearch].
					team velocityWorking: 0]]]
	} otherwise: [self printf: 'error!!!!!! in research field %c' with: field].
	^ used! !


!CqGame methodsFor: 'update' stamp: 'sma 12/19/1999 12:03'!
invest
	| newborn |
	productionYear := 0.
	self point: 33@20.
	self printf: '* investment *  '.
	
	stars do: [:star |
		star planets do: [:planet |
			(planet capacity > 10 and: [planet eSeeDef < 12])
				ifTrue: [planet eSeeDef: planet eSeeDef + 1].
			planet team notNil ifTrue: [
				newborn := (planet inhabitants * planet team growthRate *
					(1 - (planet inhabitants // planet capacity))) rounded.
				planet conquered ifTrue: [newborn := newborn // 2].
				planet inhabitants: planet inhabitants + newborn.
				planet iu: planet iu + newborn.
				
				planet team = enemy
					ifTrue: [self investEnemy: star planet: planet]
					ifFalse: [self investPlayer: star planet: planet]]]].
		
	self battle
! !

!CqGame methodsFor: 'update' stamp: 'sma 12/12/1999 01:18'!
nextYear
	turn := turn + 1.
	productionYear := productionYear + 1.

	self point: 39@18; printf: '%3d' with: turn.
	self point: 48@19; printf: '%d' with: productionYear! !

!CqGame methodsFor: 'update' stamp: 'sma 12/16/1999 20:10'!
production
	(productionYear = 4 and: [turn < 100])
		ifTrue: [self invest]! !

!CqGame methodsFor: 'update' stamp: 'sma 12/18/1999 15:39'!
revolt: star
	star planets do: [:planet |
		(planet team notNil and: [planet conquered]) ifTrue: [
			(planet team anyBc: star) ifFalse: [
				planet team: (self otherPlayer: planet team).
				planet conquered: false.
				planet pSeeCapacity: planet capacity.
				self onBoard: star position]]]! !

!CqGame methodsFor: 'update' stamp: 'sma 12/11/1999 23:53'!
updateBoard: position on: side
	| sector |
	sector := board at: position.
	self point: position x * 3 + 1 @ (16 - position y).
	console putChar: sector enemy.
	console putChar: sector star.
	console putChar: sector tf! !

!CqGame methodsFor: 'update' stamp: 'sma 12/18/1999 22:39'!
zeroTf: tf team: team
	tf exists ifFalse: [^ self].
	tf t + tf s + tf c + tf b = 0 ifFalse: [^ self].
	tf dest: nil.
	team = player ifTrue: [self onBoard: tf position]
! !


!CqGame methodsFor: 'utils' stamp: 'sma 12/11/1999 22:47'!
cursor
	^ console cursor! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/11/1999 19:52'!
getStarByName: aCharacter
	^ board starByName: aCharacter! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/16/1999 20:14'!
move: aPoint
	console move: aPoint! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/13/1999 21:29'!
otherPlayer: aPlayer
	^ player == aPlayer ifTrue: [enemy] ifFalse: [player]! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/11/1999 11:29'!
point: aPoint
	console goto: aPoint! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/11/1999 11:41'!
printf: fmtString
	console printf: fmtString! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/11/1999 11:42'!
printf: fmtString with: arg
	console printf: fmtString with: arg! !

!CqGame methodsFor: 'utils' stamp: 'sma 12/11/1999 22:47'!
printf: fmtString with: arg1 with: arg2
	console printf: fmtString with: arg1 with: arg2! !


!CqGame methodsFor: 'taskforce' stamp: 'sma 12/18/1999 22:29'!
joinTf
	"Command J(oin TF)."

	| args tf1 ch tf2 |
	self printf: 'oin tfs:'.
	args := Array with: self getLine with: 0.
	self clearLeft.
	tf1 := player tfByName: (ch := self getToken: args).
	tf1 isNil ifTrue: [^ self errorMsg: '  !! illegal tf%c' with: ch].
	tf1 exists ifFalse: [^ self errorMsg: '  !!nonexistent tf%c' with: tf1 name].
	tf1 eta = 0 ifFalse: [^ self errorMsg: '  !!tf%c is not in normal space' with: tf1 name].
	tf1 isBlasting ifTrue: [^ self errorMsg: '  !!Tf is blasting a planet    '].
 
	[tf2 := player tfByName: (ch := self getToken: args).
	ch ~= $ ] whileTrue: [
		tf2 isNil ifTrue: [self errorMsg: '  !!illegal tf%c' with: ch]
		ifFalse: [tf2 = tf1
			ifTrue: [self errorMsg: '!!Duplicate tf%c' with: tf2 name]
		ifFalse: [tf2 exists
			ifFalse: [self errorMsg: '!!Nonexistant TF%c' with: tf2 name]
		ifTrue: [tf2 position ~= tf1 position
			ifTrue: [self errorMsg: '  !!tf%c bad location' with: tf2 name]
		ifFalse: [tf2 eta ~= 0
			ifTrue: [self errorMsg: '  !!tf%c is not in normal space ' with: tf2 name]
		ifFalse: [tf2 isBlasting
			ifTrue: [self errorMsg: ' !!Tf%c is blasting a planet    ' with: tf2 name]
		ifFalse: [self joinsilent: tf1 with: tf2]]]]]]].
	self onBoard: tf1 position.
	self point: 1@19.
	self printTf: tf1
! !

!CqGame methodsFor: 'taskforce' stamp: 'sma 12/12/1999 22:43'!
joinsilent: tf1 with: tf2
	"Add all ships of second task force to first task force."

	tf1 t: tf1 t + tf2 t.
	tf1 s: tf1 s + tf2 s.
	tf1 c: tf1 c + tf2 c.
	tf1 b: tf1 b + tf2 b.
	tf2 dest: nil! !

!CqGame methodsFor: 'taskforce' stamp: 'sma 12/18/1999 22:31'!
newTf
	"Command N(ew TF)."

	| tf newTf |
	self printf: 'ew tf- from tf:'.
	tf := player tfByName: self getChar.
	self clearLeft.
	(tf isNil or: [tf exists not or: [tf eta ~= 0]]) ifTrue: [^ self errorMsg: '  !!illegal tf'].
	tf isBlasting ifTrue: [^ self errorMsg: ' !!Tf is blasting a planet'].

	self point: 1@19.
	newTf := self splitTf: tf.
	self point: 1@20.
	self printTf: newTf.
	self point: 1@21.
	self printTf: tf! !

!CqGame methodsFor: 'taskforce' stamp: 'sma 12/12/1999 22:41'!
splitTf: tf
	"Create new task force from given one."

	| newTf args ships nShips |

	newTf := player getTf: tf dest.
	
	self printf: ' ships:'.
	self point: 8 @ self cursor y.
	
	args := Array with: self getLine with: 0.
	ships := self getToken: args.
	nShips := args last.

	ships = $  ifTrue: [
		"Split entire fleet"
		newTf s: tf s; t: tf t; c: tf c; b: tf b.
		tf s: 0; t: 0; c: 0; b: 0]
	ifFalse: [
		[ships caseOf: {
			[$T] -> [
				nShips := nShips min: tf t.
				newTf t: newTf t + nShips.
				tf t: tf t - nShips].
			[$S] -> [
				nShips := nShips min: tf s.
				newTf s: newTf s + nShips.
				tf s: tf s - nShips].
			[$B] -> [
				nShips := nShips min: tf b.
				newTf b: newTf b + nShips.
				tf b: tf b - nShips].
			[$C] -> [
				nShips := nShips min: tf c.
				newTf c: newTf c + nShips.
				tf c: tf c - nShips]
		}.
		ships := self getToken: args.
		nShips := args last.
		ships = $ ] whileFalse].

	self zeroTf: tf team: player.
	self zeroTf: newTf team: player.
	self onBoard: tf position.
	^ newTf! !


!CqGame methodsFor: 'constants' stamp: 'sma 12/11/1999 11:39'!
blankLine
	^ '                              ' "30 spaces"! !

!CqGame methodsFor: 'constants' stamp: 'sma 12/19/1999 10:47'!
helpTexts
	^ #(
	(
	'B'	'Bld Battlestar(s)    75'
	'C'	'Bld Cruiser(s)       16'
	'H'	'Help'
	'R'	'Range Research'
	'S'	'Bld Scout(s)          6'
	'T'	'Bld Transports        1'
	'V'	'Velocity Research'
	'W'	'Weapons Research'
	'>M'	'Redraw Map'
	'>R'	'Research summary'
	)
	(
	'B' 'Blast Planet'           
	'C' 'Colony summary'         
	'D' 'TaskForce Destination'  
	'G' 'Go on (done)'           
	'H' 'Help'                   
	'J' 'Join TaskForces'        
	'L' 'Land transports'        
	'M' 'Redraw Map'             
	'N' 'New TaskForce'          
	'Q' 'Quit'                   
	'R' 'Research summary'       
	'S' 'Star summary'           
	'T' 'TaskForce summary'
	)
	(
	'C' 'Colonies'               
	'G' 'Go on (done)'           
	'H' 'Help'                   
	'M' 'Map'                    
	'O' 'Odds'                   
	'R' 'Research summary'       
	'S' 'Star summary'           
	'T' 'TaskForce summary'      
	'W' 'Withdraw'
	)
	(
	'B' 'Break off Attack'       
	'C' 'Colony summary'         
	'G' 'Go on (done)'           
	'H' 'Help'                   
	'J' 'Join TFs'               
	'M' 'Redraw Map'             
	'N' 'New TF'                 
	'R' 'Research summary'       
	'S' 'Star summary'           
	'T' 'TaskForce summary'
	)
	(
	'A' 'Bld Adv. Missle Base 35'
	'B' 'Bld Battlestar(s)    70'
	'C' 'Bld Cruiser(s)       16'
	'H' 'Help'                   
	'I' 'Invest                3'
	'M' 'Bld Missle Base(s)    8'
	'R' 'Range Research'         
	'S' 'Bld Scout(s)          6'
	'T' 'Bld Transports        1'
	'V' 'Vel Research'           
	'W' 'Weapons Research'       
	'>C' 'Colony summary'         
	'>M' 'Redraw Map'             
	'>R' 'Research summary'       
	'>S' 'Star summary'
	))
! !


!CqGame methodsFor: 'instructions' stamp: 'sma 4/29/2000 10:26'!
instructions
	^ 'CONQUEST INSTRUCTIONS

1.0 Introduction

    In this game, you control an interstellar empire.  You will decide
which star systems to explore, which planets to colonize, whether to spend
resources on technology, industrial development, or more combat ships.
The relative sucess of your empire depends upon your decisions.  The
computer will be directing its own empire and will be competing with you
for the best planets. Whichever of you has the greatest population at the
end of the game, will be the winner.

   At any prompt, an ''H'' command will list all possibilities for that
context.

2.0 Starting out

   You will be asked to select a starting star system.

   You start out with 30 transports (full of population) and 35 industrial
units (IU''s) to spend.  You initially can only buy ships or invest in
technology.

2.1   Ships

   There are 4 types of ships within this game.  They are listed below
along with their cost in IU''s :

      Name      Cost    Special characteristics
   ----------------------------------------------------------------
      Battlestar  75    Weapon factor 40
      Cruiser     16    Weapon factor 7
      Scout        6    Has unlimited range.
      Transport    1    Transport 1 unit of population.

2.2   Technology

   You can also invest in improving your technology.  There are three
different area which can be improved :

      Name        Description
   -----------------------------------------------------------------
      Velocity    This determines the number of units any ship can move
                  in a turn.
      Range       This determines the number of units a ship may traverse
                  without stopping at a star system.
      Weapon      This determines the effectiveness of weapons on ships or
                  missile bases.  The total weapon factor of a taskforce or
                  planetary defense system is added up and multiplied by
                  the this level.

The amount of IU''s necessary to advance to the next level varies and is
displayed in the research summary.

3.0 Game turn

   The game is divided into 100 turns (maximum).  Each turn, several things
happen.  These are as follows.

3.1   Movement

   Any task forces which have a destination are moved toward it by an
amount determined by the level of velocity technology.

3.2   Exploration

   If any task forces arrive at a star system not previously explored by
the player, it must be explored.  Exploration involves some risk.  Some
ships may be destroyed during the exploration.  The chance of this
happening depends on the type of ships in the task force.  The "best"
ship will determine the exploration risk. (i.e. a task force with a
crusier and transports is at lower risk than a task force with just
transports).

3.3   Combat

   If any task force arrive at a star system where the computer has a task
force, the forces fight till one or the other withdraws or is destroyed.
The relative chances of each task force is determined by adding the weapons
factor of all ships in the task force multiplied by the current level of
weapons research.

   If any task force arrives at a star system where the computer has a
colony, the task force may elect to try to conquer the colony.  If the
planet attacked has missile bases, the battle will be resolved in the same
manner as task force combat.

3.4   Orders

   The commands possible at this point are as follows:

3.4.1    Blasting planets

   If a planet is in your control (either uninhabited, captured, or
colonized) you may have the warships in the taskforce fire at the planet.
This will eliminate some or all of the population of the planet and
permanently reduce the population the planet can support.

3.4.2    Colony summary

   This will display a summary of the colonies currently under your control.
The display has the following format.

      Example                    Description

   G4:60(46,/49)        1amb     Star system G, planet 4, population limit
                                 60, current population 46, current
                                 industrial production 49, one advanced
                                 missile base present.

3.4.3    Taskforce destination

   This will target the specified taskforce on a particular star.  The star
must be within the range limitation specified by the range research level.

3.4.4    Go on

   This will end this turn.  All orders issued will take effect.

3.4.5    Join taskforces

   This command allows two or more taskforces to be combined into one.

3.4.6    Land transports

   If a taskforce contains any transports, and if it is in a star system
with a planet which can support more population, the player can land
transports on the planet and establish or enlarge a colony there.

3.4.7    Redraw maps

   This command causes the screen to be redrawn.

3.4.8    New taskforce

   This command allows a taskforce to be split in two.

3.4.9    Quit

   This command will terminate the game.

3.4.10   Research summary

   This command will update and display the current research levels.

3.4.11   Star summary

   This command will display a summary for the requested star systems.  The
display has the following format :
                  Example                       Comments

                  ----- star N -----            Star system designator
                  TFa         1c                Task force a - 1 crusier
                   1:10 No colony               Planet 1 - no colony
                   4:60(46/49)         1amb     Planet 4
                                                60 population limit
                                                46 population units
                                                49 industrial units
                                                1 advanced missile base

3.4.12   Taskforce summary

   This command will display a summary of the requested taskforces.  This
display has the following format.

      Example

   TFa:N( 1, 4)         1c
   TFb: ( 1, 5)      1s       R7
   TFc: ( 2, 4)      1s       M1

   Line#    Description
    1       Task force a, location star system N, coordinate (1, 4),
            contains one crusier.
    2       Task force b, coordinate (1, 5), contains one scout,
            destination star system R, arrives in 7 turns.
    3       Task force c, coordinate (2, 4), contains one scout,
            destination star system M, arrives in 1 turn.

4.0 Production turn

   Every four game turns, each planet delivers its industrial production.
This production can be spent on a number of different things.  The command
line for this mode has the following format.

      Example                 Description

    63?                       63 IU''s remain to be spent for this planet
G4:60(60,/63)        1amb     Star system G, planet 4, population limit 60,
                              current population 60, industrial production
                              63, one advanced missile base present.

As you spend IU''s, the amount remaining will be reduced.  If you buy
missile bases or invest in industrial production, the colony line will
reflect the change.  If you purchase ships, a task force will be created
and displayed below the colony line.  If you spend IU''s on research, those
research categories will be updated.

5.0 Ending the game

   There are 3 different ways to end the game.

5.1   Quitting

   You can quit at any orders prompt.  If you do, the current total
populations will be displayed and the computer will declare itself the
winner no matter which of you is ahead. (after all, it didn''t quit).

5.2   Overwhelm an opponent

   If at any point during the game, you or the computer has greater than
eight times the population that the other has, the game is ended and the
larger population is declared as the winner.

5.3   100 turns

   At the end of 100 game turns, the larger population is declared the
winner.

6.0 Credits

   Amiga port by Bob Shimbo, orginal author unknown.  Squeak Smalltalk
version by Stefan Matthias Aust.'! !

!CqGame methodsFor: 'instructions' stamp: 'sma 4/29/2000 10:43'!
showInstructions
	| lines |
	console clear; printf: 'Do you want to read the instructions (y/n)?'.
	console nextChar asUppercase = $Y ifFalse: [^ self].
	console clear.
	lines := 0.
	self instructions do:
		[:each |
		console putChar: each.
		each = Character cr
			ifTrue:
				[(lines := lines + 1) = 23
					ifTrue:
						[console cr; printf: '---MORE---'; nextChar; clear.
						lines := 0]]].
	console cr; cr; 
		printf: 'Hit return to continue';
		nextChar! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqGame class
	instanceVariableNames: ''!

!CqGame class methodsFor: 'class initialization' stamp: 'sma 12/18/1999 20:22'!
initialize
	Dice := Random new! !


!CqGame class methodsFor: 'instance creation' stamp: 'sma 12/19/1999 11:23'!
run
	^ CqConsole run: self new! !


!CqGame class methodsFor: 'utility' stamp: 'sma 12/18/1999 20:27'!
rnd: n
	^ ((Dice next) * n) floor + 1

	"((1 to: 10000) collect: [:e | CqGame rnd: 10]) asBag"! !


Object subclass: #CqPlanet
	instanceVariableNames: 'capacity team number iu mb amb conquered underAttack inhabitants pSeeCapacity eSeeTeam eSeeDef star '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqPlanet commentStamp: '<historical>' prior: 0!
A data object representing a planet of a star.!


!CqPlanet methodsFor: 'initialization' stamp: 'sma 12/18/1999 20:26'!
initialize
	capacity := (CqGame rnd: 2) > 1
		ifTrue: [(CqGame rnd: 4) + 2 * 10]
		ifFalse: [(CqGame rnd: 4) * 5].
	pSeeCapacity := capacity.
	eSeeDef := 1.
	conquered := underAttack := false.
	mb := amb := 0.
	inhabitants := iu := 0! !


!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:07'!
amb
	^ amb! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:07'!
amb: anInteger
	amb := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/1/1999 20:37'!
capacity
	^ capacity! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/18/1999 13:20'!
capacity: anInteger
	capacity := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:08'!
conquered
	^ conquered! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:08'!
conquered: aBoolean
	conquered := aBoolean! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/13/1999 21:31'!
eSeeDef
	^ eSeeDef! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/13/1999 21:32'!
eSeeDef: anInteger
	eSeeDef := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 12:24'!
eSeeTeam
	^ eSeeTeam! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 12:24'!
eSeeTeam: aPlayer
	eSeeTeam := aPlayer! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:09'!
inhabitants
	^ inhabitants! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:09'!
inhabitants: anInteger
	inhabitants := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:08'!
iu
	^ iu! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:09'!
iu: anInteger
	iu := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:07'!
mb
	^ mb! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:07'!
mb: anInteger
	mb := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:09'!
number
	^ number! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:09'!
number: anInteger
	number := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 12:23'!
pSeeCapacity
	^ pSeeCapacity! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 12:24'!
pSeeCapacity: anInteger
	pSeeCapacity := anInteger! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/18/1999 13:40'!
star
	^ star! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/18/1999 13:40'!
star: aStar
	star := aStar! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:06'!
team
	^ team! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:08'!
team: aPlayer
	team := aPlayer! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:08'!
underAttack
	^ underAttack! !

!CqPlanet methodsFor: 'accessing' stamp: 'sma 12/12/1999 00:08'!
underAttack: aBoolean
	underAttack := aBoolean! !


!CqPlanet methodsFor: 'printing' stamp: 'sma 12/1/1999 20:24'!
printOn: aStream
	aStream nextPutAll: 'planet['; print: capacity; nextPut: $]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqPlanet class
	instanceVariableNames: ''!

!CqPlanet class methodsFor: 'instance creation' stamp: 'sma 12/11/1999 19:15'!
new
	^ super new initialize! !


Object subclass: #CqPlayer
	instanceVariableNames: 'weaponsResearch weaponsWorking velocityResearch velocityWorking rangeResearch rangeWorking growthRate taskforces arrivals departures '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqPlayer commentStamp: '<historical>' prior: 0!
A data object which encapsulates player specific data.!


!CqPlayer methodsFor: 'initialization' stamp: 'sma 12/11/1999 20:09'!
initialize
	rangeResearch := 5.
	velocityResearch := 1.
	weaponsResearch := 3.
	rangeWorking := 0.
	velocityWorking := 0.
	weaponsWorking := 0.
	growthRate := 0.3.

	taskforces := (1 to: 26) collect: [:i | CqTf new name: (i + 96) asCharacter]! !


!CqPlayer methodsFor: 'printing' stamp: 'sma 12/19/1999 11:36'!
printOn: aStream
	aStream nextPutAll: (growthRate > 0.3 ifTrue: ['ENEMY'] ifFalse: ['player'])! !


!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 19:23'!
growthRate
	^ growthRate! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 19:23'!
growthRate: aFloat
	growthRate := aFloat! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:12'!
rangeReq
	^ #(0 0 0 0 0 20 40 70 100 150 200 300 400 600 900)! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:06'!
rangeResearch
	^ rangeResearch! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:06'!
rangeResearch: anInteger
	rangeResearch := anInteger! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:10'!
rangeWorking
	^ rangeWorking! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:11'!
rangeWorking: anInteger
	rangeWorking := anInteger! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:12'!
velocityReq
	^ #(0 40 60 80 120 150 200 250 300 400 500 600)
! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 19:23'!
velocityResearch
	^ velocityResearch! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 19:23'!
velocityResearch: anInteger
	velocityResearch := anInteger! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:10'!
velocityWorking
	^ velocityWorking! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:11'!
velocityWorking: anInteger
	velocityWorking := anInteger! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:08'!
weaponsReq
	^ #(0 0 0 50 70 90 120 150 250 350)
! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 19:23'!
weaponsResearch
	^ weaponsResearch! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 19:23'!
weaponsResearch: anInteger
	weaponsResearch := anInteger! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:10'!
weaponsWorking
	^ weaponsWorking! !

!CqPlayer methodsFor: 'research' stamp: 'sma 12/11/1999 20:10'!
weaponsWorking: anInteger
	weaponsWorking := anInteger! !


!CqPlayer methodsFor: 'taskforces' stamp: 'sma 12/19/1999 11:06'!
anyBc: star
	^ (self tfs detect: [:tf | tf dest = star and: [tf eta = 0 and: [tf c + tf b > 0]]] ifNone: [nil]) notNil! !

!CqPlayer methodsFor: 'taskforces' stamp: 'sma 12/18/1999 22:39'!
getTf: star
	"Answer a new task force at the specified star."

	| tf |
	tf := taskforces detect: [:each | each exists not] ifNone: [^ nil].
	tf s: 0; t: 0; c: 0; b: 0; eta: 0; origeta: 0.
	tf position: star position.
	tf dest: star.
	tf isBlasting: false.
	^ tf! !

!CqPlayer methodsFor: 'taskforces' stamp: 'sma 12/18/1999 13:51'!
tfByName: aCharacter
	| index |
	index := aCharacter asUppercase asciiValue - 64.
	^ (index between: 1 and: 26) ifTrue: [taskforces at: index]! !

!CqPlayer methodsFor: 'taskforces' stamp: 'sma 12/11/1999 23:41'!
tfs
	^ taskforces! !

!CqPlayer methodsFor: 'taskforces' stamp: 'sma 12/12/1999 00:02'!
tfsAtStar: aStar
	^ taskforces select: [:tf | tf dest = aStar and: [tf eta = 0]]! !


!CqPlayer methodsFor: 'stars' stamp: 'sma 12/12/1999 12:42'!
arrivals
	^ arrivals ifNil: [arrivals := Set new]! !

!CqPlayer methodsFor: 'stars' stamp: 'sma 12/12/1999 11:44'!
coloniesAtStar: star
	^ (star planets select: [:p | p team = self]) size! !

!CqPlayer methodsFor: 'stars' stamp: 'sma 12/12/1999 12:43'!
departures
	^ departures ifNil: [departures := Set new]! !

!CqPlayer methodsFor: 'stars' stamp: 'sma 12/12/1999 12:43'!
resetArrivals
	arrivals := nil! !

!CqPlayer methodsFor: 'stars' stamp: 'sma 12/12/1999 12:43'!
resetDepartures
	departures := nil! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqPlayer class
	instanceVariableNames: ''!

!CqPlayer class methodsFor: 'instance creation' stamp: 'sma 12/11/1999 19:31'!
new
	^ super new initialize! !


Object subclass: #CqRef
	instanceVariableNames: 'object aspect '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqRef commentStamp: '<historical>' prior: 0!
Refers to an aspect of another object to pass values by reference.!


!CqRef methodsFor: 'initialization' stamp: 'sma 12/18/1999 13:05'!
setObject: anObject aspect: aSymbol
	object := anObject.
	aspect := aSymbol! !


!CqRef methodsFor: 'accessing' stamp: 'sma 12/18/1999 13:03'!
value
	^ object perform: aspect! !

!CqRef methodsFor: 'accessing' stamp: 'sma 12/18/1999 13:04'!
value: value
	object perform: (aspect , ':') asSymbol with: value! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqRef class
	instanceVariableNames: ''!

!CqRef class methodsFor: 'instance creation' stamp: 'sma 12/18/1999 13:06'!
object: anObject aspect: aSymbol
	^ self new setObject: anObject aspect: aSymbol! !


Object subclass: #CqSector
	instanceVariableNames: 'enemy star tf '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqSector commentStamp: '<historical>' prior: 0!
A simple data object that encapsulates a map sector.!


!CqSector methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:22'!
enemy
	^ enemy! !

!CqSector methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:23'!
enemy: aCharacter
	enemy := aCharacter! !

!CqSector methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:22'!
star
	^ star! !

!CqSector methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:22'!
star: aCharacter
	star := aCharacter! !

!CqSector methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:22'!
tf
	^ tf! !

!CqSector methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:22'!
tf: aCharacter
	tf := aCharacter! !


Object subclass: #CqStar
	instanceVariableNames: 'name position planets visited '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqStar commentStamp: '<historical>' prior: 0!
A data object representing a star system of the map.!


!CqStar methodsFor: 'initialization' stamp: 'sma 12/18/1999 20:26'!
initialize
	| n |
	n := (CqGame rnd: 6) // 2.
	planets := (1 to: n) collect: [:i | CqPlanet new number: i; star: self].

	visited := Set new! !


!CqStar methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:40'!
name
	^ name! !

!CqStar methodsFor: 'accessing' stamp: 'sma 12/11/1999 15:40'!
name: aCharacter
	name := aCharacter! !

!CqStar methodsFor: 'accessing' stamp: 'sma 12/1/1999 20:36'!
planets
	^ planets! !

!CqStar methodsFor: 'accessing' stamp: 'sma 12/1/1999 20:34'!
position
	^ position! !

!CqStar methodsFor: 'accessing' stamp: 'sma 12/1/1999 20:33'!
position: aPoint
	position := aPoint! !


!CqStar methodsFor: 'printing' stamp: 'sma 12/1/1999 20:35'!
printOn: aStream
	aStream nextPutAll: 'star '; print: name; space; print: position; nextPutAll: ' [ '.
	planets do: [:each | aStream print: each; space].
	aStream nextPut: $]! !


!CqStar methodsFor: 'utility' stamp: 'sma 12/5/1999 17:53'!
dist: aStar
	^self position dist: aStar position! !


!CqStar methodsFor: 'visiting' stamp: 'sma 12/11/1999 23:39'!
beVisitedBy: aPlayer
	visited add: aPlayer! !

!CqStar methodsFor: 'visiting' stamp: 'sma 12/11/1999 23:40'!
visitedBy: aPlayer
	^ visited includes: aPlayer! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqStar class
	instanceVariableNames: ''!

!CqStar class methodsFor: 'instance creation' stamp: 'sma 12/11/1999 19:15'!
new
	^ super new initialize! !


Object subclass: #CqTf
	instanceVariableNames: 'name position eta dest isBlasting t s c b withdrew origeta positionf '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sma-Conquest'!
!CqTf commentStamp: '<historical>' prior: 0!
A data object implementing a fleet of space ships, a so called task force.!


!CqTf methodsFor: 'initialization' stamp: 'sma 12/11/1999 22:51'!
initialize
	t := s := c := b := 0.
	eta := 0.
	isBlasting := false! !


!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
b
	^ b! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
b: anInteger
	b := anInteger! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
c
	^ c! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:48'!
c: anInteger
	c := anInteger! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:36'!
dest
	^ dest! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:36'!
dest: aStar
	dest := aStar! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:36'!
eta
	^ eta! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:36'!
eta: anInteger
	eta := anInteger! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:36'!
isBlasting
	^ isBlasting! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:36'!
isBlasting: aBoolean
	isBlasting := aBoolean! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:35'!
name
	^ name! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:35'!
name: aCharacter
	name := aCharacter! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/12/1999 10:42'!
origeta
	^ origeta! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/12/1999 10:42'!
origeta: anInteger
	origeta := anInteger! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:35'!
position
	^ position! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:35'!
position: aPoint
	position := aPoint! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/12/1999 10:55'!
positionf
	^ positionf! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/12/1999 10:55'!
positionf: aPoint
	positionf := aPoint! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
s
	^ s! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
s: anInteger
	s := anInteger! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
t
	^ t! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/11/1999 19:47'!
t: anInteger
	t := anInteger! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/12/1999 10:42'!
withdrew
	^ withdrew! !

!CqTf methodsFor: 'accessing' stamp: 'sma 12/12/1999 10:43'!
withdrew: aBoolean
	withdrew := aBoolean! !


!CqTf methodsFor: 'printing' stamp: 'sma 12/19/1999 11:32'!
printOn: aStream
	self exists
		ifTrue:
			[aStream nextPutAll: 'tf '; print: position; nextPutAll: ' ['.
			t > 0 ifTrue: [aStream space; print: t; nextPut: $T].
			s > 0 ifTrue: [aStream space; print: s; nextPut: $S].
			c > 0 ifTrue: [aStream space; print: c; nextPut: $C].
			b > 0 ifTrue: [aStream space; print: b; nextPut: $B].
			aStream nextPutAll: ' ]'.
			eta = 0 
				ifTrue: [aStream nextPutAll: ' at ']
				ifFalse: [aStream space; print: eta; nextPutAll: ' to '].
			aStream nextPut: dest name]
		ifFalse: [aStream nextPutAll: 'nonexistant tf']! !


!CqTf methodsFor: 'testing' stamp: 'sma 12/18/1999 22:24'!
exists
	^ dest notNil! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CqTf class
	instanceVariableNames: ''!

!CqTf class methodsFor: 'instance creation' stamp: 'sma 12/11/1999 22:50'!
new
	^ super new initialize! !

CqConsole initialize!
CqGame initialize!

CqGame run!